@@ -1,15 +1,15 @@
-/*** analhea2.h; header file for analog version 1.91beta4 ***/
+/*** analhea2.h; header file for analog version 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/ */
 
-/*** These things are not intended to be user definable; user definable headers
-     are in analhead.h ***/
+/*** These things are not intended to be user definable;
+     user definable headers are in analhead.h ***/
 
 #ifndef ANALHEAD2
 #define ANALHEAD2
 
 /*** First some #define's and #include's. Start with system specific ones ***/
 
-#define VNUMBER "1.91beta4"    /* the version number of this program */
+#define VNUMBER "2.0"    /* the version number of this program */
 
 #ifdef __MWERKS__
 #define MAC        /* So MWERKS compiler automatically switches to Mac */
@@ -22,9 +22,9 @@
 #ifdef MAC
 #define MAC_EVENTS
 #define WEBSTAR
+#define NETPRESENZ
 #define NOPIPES
-#define NODNS
-#define NODIRENT
+#define MACDIRENT
 #include <MacHeaders.h>
 #include <sioux.h>
 #define VERSION VNUMBER "/Mac"
@@ -49,6 +49,11 @@
 #define DIRSEP '/'
 #endif
 
+#ifdef WIN32
+#define VERSION VNUMBER "/Win32"
+#define DIRSEP '\\'
+#endif
+
 /* If no OS is defined, the compilation will fail by not knowing VERSION */
 
 #include <stdio.h>
@@ -59,16 +64,26 @@
 #include <ctype.h>
 #ifndef NODIRENT
 #ifndef VMSDIRENT
-#include <dirent.h>
+#ifndef MACDIRENT
+#include <dirent.h>          /* Normal dirent */
 #include <sys/types.h>
 #include <sys/stat.h>
 #else
-#define VMS_FSPEC_MAX 256
+#include "macdir.h"          /* Mac dirent */
+#endif
+#else
+#define VMS_FSPEC_MAX 256    /* VMS dirent */
 #include <descrip.h>
 #endif
 #endif
 #ifndef NODNS
-#ifndef MAC
+#ifdef MAC
+#include <Gestalt.h>
+#include <MacTCP.h>
+#include <OpenTransport.h>
+#include <OpenTptInternet.h>
+#include <AddressXlation.h>
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -76,6 +91,9 @@
 #define INET_ADDR_ERR (-1)   /* what inet_addr() returns on error */
 #endif
 #endif
+#ifdef MAC_EVENTS
+#define MAC_IDLE_FREQ (200)
+#endif
 
 #ifdef TRUE  /* I suppose everyone else defines TRUE = 1, but I won't */
 #undef TRUE  /* risk it by doing an #ifndef! */
@@ -121,10 +139,22 @@
 #undef GERMAN
 #endif
 #define GERMAN (2)
-#ifdef US_ENGLISH           /* output languages */
+#ifdef ITALIAN
+#undef ITALIAN
+#endif
+#define ITALIAN (3)
+#ifdef SPANISH
+#undef SPANISH
+#endif
+#define SPANISH (4)
+#ifdef US_ENGLISH
 #undef US_ENGLISH
 #endif
-#define US_ENGLISH (4)
+#define US_ENGLISH (5)
+#ifdef DANISH
+#undef DANISH
+#endif
+#define DANISH (6)
 #define SENSITIVE (0)    /* as in, "Case ... filesystem" */
 #define INSENSITIVE (1)
 #define BYREQUESTS (0)       /* ways of sorting */
@@ -133,8 +163,9 @@
 #define RANDOMLY (3)
 #define BYPAGES (4)
 #define APPROX (2)       /* for hostname counting */
-#define WEBSTARLINE (3)  /* four types of input lines */
-#define COMMON (2)       /* (in fact we only ever test whether zero or not */
+#define NETPRESENZLINE (4)
+#define WEBSTARLINE (3)
+#define COMMON (2)
 #define NCSAOLD (1)
 #define CORRUPT (0)
 #define PREFORMATTED (3)   /* output types */
@@ -168,6 +199,13 @@
 #define ISLEAPFEB(m,y) (((m)==1)&&(((y)%4)==0))  /* Feb in leap year */
 #define ISLEAPJF(m,y) (((m)<=1)&&(((y)%4)==0))  /* Jan or Feb in leap year */
 /* NB Every 4th year from 1901 to 2099 is a leap year */
+#define MAGICNO(x,s,b) {register unsigned char *r; \
+			for(x = 0, r = (unsigned char *)s; *r != '\0'; r++) { \
+			  x = (x << 1) + (int)(*r); \
+			  while (x >= (b)) \
+			    x -= b; }}
+/* magicno as macro not fn. for speed, tho' appears to make little diff. */
+/* x is answer, s is string to be magicked, b is base (no, of hash bins) */
 
 #define DOMHASHSIZE (1354)
 /* = 2 * 26^2 + 2 by description of domain algorithm */
@@ -240,7 +278,7 @@ struct genstruct {           /* relevant information about an object */
 struct dnscache {            /* information about DNS lookups */
   char *number;              /* the numerical IP address */
   char *alias;               /* the translation (or NULL if unresolvable) */
-  long altimecode;           /* the time at which its alias was looked up */
+  time_t altimecode;         /* the time at which its alias was looked up */
   struct dnscache *next;
 };
 
@@ -301,13 +339,16 @@ extern flag itemwanted(char *name, flag ispage, char codeletter);
 /* in formgen.c */
 extern void genopts(FILE *outf, char name[17], char plural[16], int sortby,
   char codeletter);
+extern void repchoice(FILE *outf, char name[MAXSTRINGLENGTH], char codeletter,
+  flag qq);
 extern void formgen(void);
 /* in hash.c */
 extern struct genstruct *hashadd(struct genstruct **objhead, int hashsize,
   char *name, int reqs, double bytes, int pages, flag last7q, int *totalobjs,
   int *totalobjs7, int *totalnew7, flag al, flag maskq, flag ispage,
   struct genstruct *freespace, int magicnumber, char codeletter);
-extern struct dnscache *dnshashadd(char *number, char *alias, long altimecode);
+extern struct dnscache *dnshashadd(char *number, char *alias,
+  time_t altimecode);
 extern void domhashadd(char *hostn, int reqs, int pages, double bytes);
 extern void subdomadd(char *id, char *name);
 extern void addref(char *fromurl, char *filename, flag ispage, double bytes,
@@ -407,6 +448,12 @@ extern void MacInit(void);
 extern void MacFini(void);
 extern void MacIdle(void);
 #endif
+#ifdef MAC
+#ifndef NODNS
+int IpAddr2Name(char *hostname);
+void ResolverCleanup(void);
+#endif
+#endif
 /* in output.c */
 extern void output(struct genstruct *rsorthead, struct genstruct *isorthead,
   struct genstruct *tsorthead, struct genstruct *Ssorthead, int firstdom,
@@ -419,23 +466,26 @@ extern void htmlfprintf(FILE *outf, char string[MAXSTRINGLENGTH]);
 extern double finddivider(double bytes, char *bprefix);
 extern void asciiline(FILE *outf);
 extern void barplot(FILE *outf, int n);
-extern void precols(FILE *outf, char *wantcols, char codeletter, flag byq);
+extern void precols(FILE *outf, char *wantcols, char codeletter, flag byq,
+  flag pageq);
 extern void printcolheads(FILE *outf, char *wantcols, int fieldwidth,
   int pfieldwidth, int bfieldwidth, char bprefix[2], char name[20], char type,
-  flag byq, flag name1st);
+  flag byq, flag pageq, flag name1st);
 extern void printcols(FILE *outf, char *wantcols, int reqs, int pages,
   double bytes, int fieldwidth, int pfieldwidth, int bfieldwidth,
   double bdivider, int totreqs, int totpages, double totbytes, char type,
-  flag byq);
+  flag byq, flag pageq);
 extern int whatincluded(FILE *outf, int sortby, char *minreqstr,
   char *minpagestr, char *minbytestr, char singular[27], char plural[29],
   flag subdoms, char gender);
+extern int whatincludednop(int sortby, char *minreqstr, char *minpagestr,
+  char *minbytestr);
 extern void genout(FILE *outf, struct genstruct *sorthead, int tot_reqs,
   int tot_pages, double tot_bytes, int sortby, char *minreqstr,
   char *minpagestr, char *minbytestr, int max_reqs, int max_pages,
   double max_bytes, char *wantcols, char anchor[10], char title[36],
   char singular[22], char plural[24], char colhead[24], char gender,
-  char codeletter, flag alphahost, flag byq, struct include *links,
+  char codeletter, flag alphahost, flag byq, flag pageq, struct include *links,
   struct alias *aka, char baseurl[MAXSTRINGLENGTH]);
 extern void domout(FILE *outf, int firstdom);
 extern void datehead(FILE *outf, int maxreq, int maxpages, double maxbytes,
@@ -473,6 +523,22 @@ extern int sscanf_webstar(char *inputline, char hostn[MAXSTRINGLENGTH],
   char filename[MAXSTRINGLENGTH], char referrer[MAXSTRINGLENGTH],
   char agent[MAXSTRINGLENGTH], int *code, char bytestr[16], size_t preflength);
 #endif
+#ifdef NETPRESENZ
+typedef enum { NP_HTTP, NP_GOPHER, NP_ADMIN, NP_FTP, NP_GETFILE, NP_GETDIR,
+		 NP_LOGIN, NP_LOGOUT, NP_CGI, NP_ERROR } NP_type;
+typedef struct { char *str; NP_type value; } MatchPairs;
+NP_type checkValue(char *typeStr, const MatchPairs table[]);
+NP_type checkType(char *typeStr);
+void build_agent(char agent[MAXSTRINGLENGTH], NP_type entryType,
+  char *entryTypeStr, NP_type actionType, char *actionTypeStr);
+int readDigits(char **from);
+int scan_path(char *inputline, char filename[MAXSTRINGLENGTH], int maxlen,
+  struct include *expandhead);
+int sscanf_netpresenz(FILE *logfile, char *inputline,
+  char hostn[MAXSTRINGLENGTH], int *date, int *monthno, int *year, int *hr,
+  int *min, char filename[MAXSTRINGLENGTH], char referer[MAXSTRINGLENGTH],
+  char agent[MAXSTRINGLENGTH], int *code, char bytestr[16], size_t preflength);
+#endif
 /* in utils.c */
 extern flag matchq(char *string, char *pattern, flag whole);
 extern flag wildmatch(char *string, char *pattern, char **w1, char **w2);
@@ -492,7 +558,6 @@ extern void *xcalloc(size_t nelem, size_t elsize);
 extern char *strtolower(char *string);
 extern flag strtolowerf(char *string);
 extern char *strtoupper(char *string);
-extern unsigned int magicno(char *string, int base);
 extern int hoststrcmp(char *hostn1, char *hostn2);
 extern flag isnumeric(char *name);
 #endif  /* not previously included */
