@@ -1,4 +1,4 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** analog.c; the main function ***/
@@ -24,7 +24,7 @@ int main(int argc, char **argv)
   extern flag eq;
   extern int osortby, rsortby, isortby, tsortby, Ssortby, fsortby;
   extern int bsortby, Bsortby;
-  extern char bcols[], Bcols[];
+  extern char bcols[], Bcols[], fcols[];
   extern char mgraph, dgraph, Dgraph, hgraph, Hgraph, Wgraph;
   extern char *rminreqstr, *iminreqstr, *tminreqstr, *Sminreqstr;
   extern char *fminreqstr, *bminreqstr, *Bminreqstr;
@@ -59,7 +59,7 @@ int main(int argc, char **argv)
   extern int errors[];
   extern int onumber;
   extern struct include *wanthosthead, *wantreqhead;
-  extern int debug;
+  extern int debug, progressfreq;
   extern int status[], status7[], statusnos[];
 
   FILE *lf;
@@ -79,8 +79,9 @@ int main(int argc, char **argv)
   size_t preflength;     /* length of filename prefix for this logfile */
   flag firstreq = TRUE;
   flag datemaskq;
-  flag fwarn1 = OFF, fwarn2 = OFF;   /* have certain warnings been given? */
+  flag fwarn1 = OFF, fwarn2 = OFF, fwarn3 = OFF;
   flag bwarn1 = OFF, bwarn2 = OFF, bwarn3 = OFF, bwarn4 = OFF;
+  /* have certain warnings been given? */
   double bytes;    /* long is not big enough; double has more sig. figs,
 		      and copes with overflow automatically. */
   char bytestr[16];
@@ -97,6 +98,8 @@ int main(int argc, char **argv)
   flag ispage = OFF;        /* whether it represents a page */
   flag last7q = OFF;        /* are we now in the last 7 days? */
   int rc;
+  register int linesread;
+  int nextreport;
   char *tempstr, *tempc;
   int i, tempint, tempint2;
   flag tempf;
@@ -106,12 +109,11 @@ int main(int argc, char **argv)
   /*** Initialisation ***/
 
 #ifdef MAC_EVENTS
-#define MAC_IDLE_FREQ (200)
-#define MAC_REPORT_FREQ (5000)  /* MRF must be a multiple of MIF !! */
-  register int Maclines = 0;
   MacInit();
 #endif
   initialise(argc, argv);
+  linesread = 0;
+  nextreport = progressfreq;
   datemaskq = (fromtime.code > -INFINITY || totime.code < INFINITY);
 
   /*** Now start scanning ***/
@@ -122,18 +124,13 @@ int main(int argc, char **argv)
     lf = fopenlog(logfilep -> name, "logfile", &ispipe);
     if (lf != NULL) {
 
-      strcpy(filename, logfilep -> prefix);
-      preflength = strlen(filename);
+      preflength = strlen(logfilep -> prefix);
 
       while(fgets(inputline, MAXLINELENGTH, lf) != NULL) {
 
-#ifdef MAC_EVENTS
-	if (++Maclines % MAC_IDLE_FREQ == 0) {
-	  if (++Maclines % MAC_REPORT_FREQ == 0)
-	    printf("%10d lines read\n", Maclines);
-	  MacIdle();
-	}
-#endif
+	strcpy(filename, logfilep -> prefix);
+	/* needed each line coz can get accidentally overwritten by /../ */
+
 	linetype = CORRUPT;   /* paranoia :) */
 
 	if ((rc = sscanf_common(inputline, hostn, &date, &monthno, &year, &hr,
@@ -151,13 +148,17 @@ int main(int argc, char **argv)
 	  bytes = atof(bytestr);
 	}
 #endif
-	else if ((rc = sscanf_ncsaold(inputline, hostn, &monthno, &date, &hr,
-				      &min, &year, filename + preflength,
-				      preflength)) == 7) {
-	  linetype = NCSAOLD;
-	  code = 200;
+#ifdef NETPRESENZ
+	else if ((rc = sscanf_netpresenz(lf, inputline, hostn, &date, &monthno,
+					 &year, &hr, &min,
+					 filename + preflength, fromurl,
+					 browser, &code, bytestr,
+					 preflength)) >= 9) {
+	  linetype = NETPRESENZLINE;
 	  bytes = 0;
 	  if (byq) {
+	    fprintf(stderr, "%s: Warning: Netpresenz logs contain no bytes information:\n", commandname);
+	    fprintf(stderr, "  will not report on bytes transferred.\n");
 	    byq = OFF;
 	    if (osortby == BYBYTES)
 	      osortby = BYREQUESTS;
@@ -165,6 +166,43 @@ int main(int argc, char **argv)
 	      rsortby = BYREQUESTS;
 	    if (isortby == BYBYTES)
 	      isortby = BYREQUESTS;
+	    if (tsortby == BYBYTES)
+	      tsortby = BYREQUESTS;
+	    if (Ssortby == BYBYTES)
+	      Ssortby = BYREQUESTS;
+	    if (mgraph == 'b' || mgraph == 'B')
+	      mgraph = 'r';
+	    if (dgraph == 'b' || dgraph == 'B')
+	      dgraph = 'r';
+	    if (Dgraph == 'b' || Dgraph == 'B')
+	      Dgraph = 'r';
+	    if (hgraph == 'b' || hgraph == 'B')
+	      hgraph = 'r';
+	    if (Hgraph == 'b' || Hgraph == 'B')
+	      Hgraph = 'r';
+	    if (Wgraph == 'b' || Wgraph == 'B')
+	      Wgraph = 'r';
+	  }
+	}
+#endif
+	else if ((rc = sscanf_ncsaold(inputline, hostn, &monthno, &date, &hr,
+				      &min, &year, filename + preflength,
+				      preflength)) == 7) {
+	  linetype = NCSAOLD;
+	  code = 200;
+	  bytes = 0;
+	  if (byq) {
+	    fprintf(stderr, "%s: Warning: old style NCSA logs contain no bytes information:\n", commandname);
+	    fprintf(stderr, "  will not report on bytes transferred.");
+	    byq = OFF;
+	    if (osortby == BYBYTES)
+	      osortby = BYREQUESTS;
+	    if (rsortby == BYBYTES)
+	      rsortby = BYREQUESTS;
+	    if (isortby == BYBYTES)
+	      isortby = BYREQUESTS;
+	    if (tsortby == BYBYTES)
+	      tsortby = BYREQUESTS;
 	    if (Ssortby == BYBYTES)
 	      Ssortby = BYREQUESTS;
 	    if (mgraph == 'b' || mgraph == 'B')
@@ -295,7 +333,7 @@ int main(int argc, char **argv)
 	    }
 
 	  }  /* end if want this one */
-	
+	  
 	}   /* end if linetype != CORRUPT */
 
 	else {   /* line is corrupt */
@@ -310,6 +348,16 @@ int main(int argc, char **argv)
 	  }
 	}
   
+	if ((++linesread) == nextreport) {
+	  fprintf(stderr, "%s: %d lines read\n", commandname, linesread);
+	  nextreport += progressfreq;
+	}
+
+#ifdef MAC_EVENTS
+	if ((linesread % MAC_IDLE_FREQ) == 0)
+	  MacIdle();
+#endif
+
       }   /* end of reading this logfile */
 
       fcloselog(lf, logfilep -> name, "logfile", ispipe);
@@ -474,22 +522,45 @@ int main(int argc, char **argv)
 	      }
 	    }
 	    if (!fwarn2 && hostmaskq && wantthisone && warnq) {
-	      fprintf(stderr, "%s: Warning: Referrer log contains no host information;\n", commandname);
+	      fprintf(stderr, "%s: Warning: Referrer logs contain no host information;\n", commandname);
 	      fprintf(stderr, "  cannot apply HOSTINCLUDE and HOSTEXCLUDE commands to it.\n");
 	      fwarn2 = ON;
 	      anywarns = ON;
 	    }
-	    if (wantthisone) {
-	      refbyq = OFF;
-	      if (fsortby == BYBYTES)
-		fsortby = BYREQUESTS;
-	      addref(fromurl, filename, UNSET, 0.0, last7q, filemaskq);
+	    if (!fwarn3) {
+	      fwarn3 = ON;
+	      for (tempc = fcols; *tempc != '\0' && *tempc != 'B' &&
+		   *tempc != 'b'; tempc++)
+		;
+	      if (*tempc != '\0' || fsortby == BYBYTES) {
+		if (warnq) {
+		  fprintf(stderr, "%s: Warning: Referrer logs contain no bytes information;\n", commandname);
+		  fprintf(stderr, "  cannot report on referrer bytes.\n");
+		  anywarns = ON;
+		}
+		if (fsortby == BYBYTES)
+		  fsortby = BYREQUESTS;
+		refbyq = OFF;
+	      }
 	    }
+	    if (wantthisone)
+	      addref(fromurl, filename, UNSET, 0.0, last7q, filemaskq);
 	    else
 	      ++total_masked_refs;
 	  }
 	  else
 	    ++total_bad_refs;
+
+	if ((++linesread) == nextreport) {
+	  fprintf(stderr, "%s: %d lines read\n", commandname, linesread);
+	  nextreport += progressfreq;
+	}
+
+#ifdef MAC_EVENTS
+	if ((linesread % MAC_IDLE_FREQ) == 0)
+	  MacIdle();
+#endif
+
 	}
 	
 	fcloselog(lf, otherlogp -> name, "referrer log", ispipe);
@@ -547,48 +618,57 @@ int main(int argc, char **argv)
 	    }
 	    if (warnq) {  /* some boring warnings */
 	      if (!bwarn2 && hostmaskq && wantthisone) {
-		fprintf(stderr, "%s: Warning: Browser log contains no host information;\n", commandname);
-		fprintf(stderr, "  cannot apply HOSTINCLUDE and HOSTEXCLUDE commands to it.\n");
+		fprintf(stderr, "%s: Warning: Browser logs contain no host information;\n", commandname);
+		fprintf(stderr, "  cannot apply HOSTINCLUDE and HOSTEXCLUDE commands to them.\n");
 		bwarn2 = ON;
 		anywarns = ON;
 	      }
 	      if (!bwarn3 && filemaskq && wantthisone) {
-		fprintf(stderr, "%s: Warning: Browser log contains no file information;\n", commandname);
-		fprintf(stderr, "  cannot apply FILEINCLUDE and FILEEXCLUDE commands to it.\n");
+		fprintf(stderr, "%s: Warning: Browser logs contain no file information;\n", commandname);
+		fprintf(stderr, "  cannot apply FILEINCLUDE and FILEEXCLUDE commands to them.\n");
 		bwarn3 = ON;
 		anywarns = ON;
 	      }
-	      if (!bwarn4) {
+	    }
+	    if (!bwarn4) {
+	      bwarn4 = ON;
+	      tempf = OFF;
+	      if (bq && (bsortby == BYBYTES || bsortby == BYPAGES))
+		tempf = ON;
+	      else if (Bq && (bsortby == BYBYTES || bsortby == BYPAGES))
+		tempf = ON;
+	      else if (bq) {
 		for (tempc = bcols; *tempc != '\0' && *tempc != 'p' &&
-		     *tempc != 'P'; tempc++)
+		     *tempc != 'P' && *tempc != 'B' && *tempc != 'b';
+		     tempc++)
 		  ;
-		if (*tempc != '\0' && bq) {
-		  fprintf(stderr, "%s: Warning: Browser log contains no file information;\n", commandname);
-		  fprintf(stderr, "  browser page counts will be too low.\n");
+		if (*tempc != '\0')
+		  tempf = ON;
+	      }
+	      if (!tempf && Bq) {
+		for (tempc = Bcols; *tempc != '\0' && *tempc != 'p' &&
+		     *tempc != 'P' && *tempc != 'B' && *tempc != 'b';
+		     tempc++)
+		  ;
+		if (*tempc != '\0')
+		  tempf = ON;
+	      }
+	      if (tempf) {
+		if (warnq) {
+		  fprintf(stderr, "%s: Warning: Browser logs contain no file information;\n", commandname);
+		  fprintf(stderr, "  cannot report on browser page counts or bytes.\n");
 		  anywarns = ON;
 		}
-		else {
-		  for (tempc = Bcols; *tempc != '\0' && *tempc != 'p' &&
-		       *tempc != 'P'; tempc++)
-		    ;
-		  if (*tempc != '\0' && Bq) {
-		    fprintf(stderr, "%s: Warning: Browser log contains no file information;\n", commandname);
-		    fprintf(stderr,
-			    "  browser page counts will be too low.\n");
-		  anywarns = ON;
-		  }
-		}
-		bwarn4 = ON;
+		browbyq = OFF;
+		if (bsortby == BYBYTES || bsortby == BYPAGES)
+		  bsortby = BYREQUESTS;
+		if (Bsortby == BYBYTES || Bsortby == BYPAGES)
+		  Bsortby = BYREQUESTS;
 	      }
 	    }
 	    if (wantthisone) {
 	      if ((tempc = strchr(tempstr, '\n')) != NULL)
 		*tempc = '\0';
-	      browbyq = OFF;
-	      if (bsortby == BYBYTES)
-		bsortby = BYREQUESTS;
-	      if (Bsortby == BYBYTES)
-		Bsortby = BYREQUESTS;
 	      strcpy(browser, tempstr);
 	      addbrowser(browser, OFF, 0.0, last7q);
 	    }
@@ -598,6 +678,17 @@ int main(int argc, char **argv)
 	  }
 	  else    /* had bad date */
 	    ++total_bad_brows;
+
+	  if ((++linesread) == nextreport) {
+	    fprintf(stderr, "%s: %d lines read\n", commandname, linesread);
+	    nextreport += progressfreq;
+	  }
+
+#ifdef MAC_EVENTS
+	  if ((linesread % MAC_IDLE_FREQ) == 0)
+	    MacIdle();
+#endif
+
 	}
 
 	fcloselog(lf, otherlogp -> name, "browser log", ispipe);
@@ -637,8 +728,8 @@ int main(int argc, char **argv)
 	  if (wantthisone) {
 	    if (datemaskq) {
 	      thistimecode = timecode(date, monthno, year, hr, min);
-	      wantthisone = thistimecode > fromtime.code &&
-		thistimecode < totime.code;
+	      wantthisone = (thistimecode > fromtime.code) &&
+		(thistimecode < totime.code);
 	    }
 	    if (wantthisone) {
 	      strcpy(errstr, tempstr);
@@ -646,6 +737,16 @@ int main(int argc, char **argv)
 	    }
 	  }
 
+	  if ((++linesread) == nextreport) {
+	    fprintf(stderr, "%s: %d lines read\n", commandname, linesread);
+	    nextreport += progressfreq;
+	  }
+
+#ifdef MAC_EVENTS
+	  if ((linesread % MAC_IDLE_FREQ) == 0)
+	    MacIdle();
+#endif
+
 	}
 
 	fcloselog(lf, otherlogp -> name, "error log", ispipe);
@@ -654,6 +755,48 @@ int main(int argc, char **argv)
     }
   }
 
+  /* Now writing out the DNS cache */
+
+#ifndef NODNS
+  if (dnsq) {
+    if (STREQ(dnsfile, "-") || STREQ(dnsfile, "stdin")) {
+      lf = stdout;
+      if (debug > 0)
+	fprintf(stderr, "F: Opening stdout as DNS cache output file\n");
+    }
+    else if ((lf = fopen(dnsfile, "w")) == NULL) {
+      if (warnq) {
+	fprintf(stderr,
+		"%s: Warning: failed to open DNS cache file %s for writing.\n",
+		commandname, dnsfile);
+	anywarns = ON;
+      }
+    }
+    else if (debug > 0)
+      fprintf(stderr, "F: Opening %s as DNS cache output file\n", dnsfile);
+    if (lf != NULL) {
+      onlist = 0;
+      for (dnsp = dnshead[0]; onlist < dnshashsize; dnsp = dnsnextp) {
+	                                /* run through hosts, as below */
+	if (dnsp -> number == NULL) {
+	  dnsnextp = dnshead[++onlist];
+	}
+	else {
+	  if (isnumeric(dnsp -> number))
+	    fprintf(lf, "%d %s %s\n", dnsp -> altimecode, dnsp -> number,
+	            (dnsp -> alias == NULL)?"*":(dnsp -> alias));
+	  dnsnextp = dnsp -> next;
+	}
+      }
+      fclose(lf);
+      if (debug > 0)
+	fprintf(stderr, "F: Closing %s\n",
+		(STREQ(dnsfile, "-") || STREQ(dnsfile, "stdin"))?"stdout":
+		dnsfile);
+    }
+  }
+#endif
+
   /* now start final accounting */
 
   for (i = 0; i < NO_STATUS; i++) {
@@ -700,35 +843,6 @@ int main(int argc, char **argv)
     if (total_succ_reqs7 == 0)
       q7 = OFF;   /* just total_bytes no good in case (!byq) */
 
-    /* Now writing out the DNS cache */
-
-#ifndef NODNS
-    if (dnsq) {
-      if ((lf = fopen(dnsfile, "w")) == NULL) {
-	if (warnq)
-	  fprintf(stderr, "%s: Warning: failed to open DNS cache file %s for writing.\n",
-		  commandname, dnsfile);
-	anywarns = ON;
-      }
-      else {
-	onlist = 0;
-	for (dnsp = dnshead[0]; onlist < dnshashsize; dnsp = dnsnextp) {
-	                                   /* run through hosts, as below */
-	  if (dnsp -> number == NULL) {
-	    dnsnextp = dnshead[++onlist];
-	  }
-	  else {
-	    if (isnumeric(dnsp -> number))
-	      fprintf(lf, "%ld %s %s\n", dnsp -> altimecode, dnsp -> number,
-	            (dnsp -> alias == NULL)?"*":(dnsp -> alias));
-	    dnsnextp = dnsp -> next;
-	  }
-	}
-	fclose(lf);
-      }
-    }
-#endif
-
     /* Next do aliasing of hosts */
 
     if (sq == ON)
