@@ -1,4 +1,4 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 #include "analhea2.h"
@@ -26,7 +26,7 @@ struct genstruct *hashadd(struct genstruct **objhead, int hashsize,
   /* First calculate name's "magic number" */
 
   if (magicnumber < 0)
-    magicnumber = (int)magicno(name, hashsize);
+    MAGICNO(magicnumber, name, hashsize);
 
   /* Now look through the magicnumber'th list for that object */
 
@@ -159,21 +159,23 @@ struct genstruct *hashadd(struct genstruct **objhead, int hashsize,
 #ifndef NODNS
 /* Add an address to (or look up in) the DNS hash table. Based on hashadd()
    but much smaller. Return a pointer to the object. */
-/* NB: Changes "number" to its alias. Use altimecode = -1 for uncached. */
+/* NB: Changes "number" to its alias. */
+/* Use altimecode = (time_t)(-1) for uncached. */
 /* Assumes dnsq and isdigit(hostn[0]) already tested. */
-struct dnscache *dnshashadd(char *number, char *alias, long altimecode)
+struct dnscache *dnshashadd(char *number, char *alias, time_t altimecode)
 {
   extern struct dnscache **dnshead;
-  extern int dnshashsize;
+  extern size_t dnshashsize;
   extern time_t starttime;
 
   flag finished;
   struct dnscache *p, *lastp, *prevp;
-  int magicnumber, tempint;
+  int magicnumber;
+  int tempint;
 
   /* First calculate the "magic number" */
 
-  magicnumber = (int)magicno(number, dnshashsize);
+  MAGICNO(magicnumber, number, dnshashsize);
 
   /* Now look through the magicnumber'th list for that host */
 
@@ -197,7 +199,7 @@ struct dnscache *dnshashadd(char *number, char *alias, long altimecode)
   if (!finished) {   /* reached the end of the list without success; new one */
     p -> number = (char *)xmalloc(strlen(number) + 1);
     strcpy(p -> number, number);
-    if (altimecode >= 0) {
+    if (altimecode != (time_t)(-1)) {
       if (alias != NULL) {
 	p -> alias = (char *)xmalloc(strlen(alias) + 1);
 	strcpy(p -> alias, alias);
@@ -215,7 +217,7 @@ struct dnscache *dnshashadd(char *number, char *alias, long altimecode)
 	p -> alias = (char *)xmalloc(strlen(number) + 1);
 	strcpy(p -> alias, number);
       }
-      p -> altimecode = (long)starttime;
+      p -> altimecode = starttime;
     }
     p -> next = (struct dnscache *)xmalloc(sizeof(struct dnscache));
     p -> next -> number = NULL;
@@ -281,7 +283,7 @@ void domhashadd(char *hostn, int reqs, int pages, double bytes)
 	tempp--;
 	while (tempp != hostn && *(tempp - 1) != '.')
 	  tempp--;
-	magicnumber = (int)magicno(tempp, Ohashsize);
+	MAGICNO(magicnumber, tempp, Ohashsize);
 	finished = OFF;
 	for (domp = Ohead[magicnumber]; domp -> name != NULL && !finished;
 	     domp = domp -> next) {
@@ -325,7 +327,7 @@ void domhashadd(char *hostn, int reqs, int pages, double bytes)
       if (tempp != hostn) {
 	tempchar = *tempp;
 	*tempp = '\0';   /* temporarily trucate the string again */
-	magicnumber = (int)magicno(hostn, Ohashsize);
+	MAGICNO(magicnumber, hostn, Ohashsize);
 	finished = OFF;
 	for (domp = Ohead[magicnumber]; domp -> name != NULL && !finished;
 	     domp = domp -> next) {
@@ -385,7 +387,7 @@ void subdomadd(char *id, char *name)
   }
 
   else {
-    magicnumber = (int)magicno(id, Ohashsize);
+    MAGICNO(magicnumber, id, Ohashsize);
     domp = Ohead[magicnumber];
 
     /* look through that list and slot it in alphabetically
@@ -495,48 +497,58 @@ void addbrowser(char *browser, flag ispage, double bytes, flag last7q)
   extern int total_good_brows, total_brow_pages;
   extern double total_brow_bytes;
   extern size_t Bhashsize, bhashsize;
+  extern struct alias *browaliashead;
 
   char *c;
   int tempint;
 
-  /* cut (illegal) "via"s (e.g., via proxy gateway or via Harvest cache) */
-  if ((c = strstr(browser, " via ")) != NULL) {
-    while (*c == ' ' && c != browser)
-      c--;
-    *(c + 1) = '\0';
-  }
+  /* if the browser name is just "-", ignore it */
 
-  /* cut trailing spaces */
-  for (c = browser + strlen(browser); *(c - 1) == ' ' && c > browser; c--)
-    *(c - 1) =  '\0';
+  if (!STREQ(browser, "-")) {
 
-  ++total_good_brows;
-  total_brow_pages += ispage;
-  total_brow_bytes += bytes;
+    /* cut (illegal) "via"s (e.g., via proxy gateway or via Harvest cache) */
+    if ((c = strstr(browser, " via ")) != NULL) {
+      while (*c == ' ' && c != browser)
+	c--;
+      *(c + 1) = '\0';
+    }
 
-  /* add to full browser report */
-  if (Bq)
-    hashadd(Bhead, Bhashsize, browser, 1, bytes, ispage, last7q,
-	    &tempint, &tempint, &tempint, ON, OFF, OFF,
-	    (struct genstruct *)NULL, -1, 'B');
+    /* cut trailing spaces */
+    for (c = browser + strlen(browser); *(c - 1) == ' ' && c > browser; c--)
+      *(c - 1) =  '\0';
 
-  if (bq) {
+    doaliaslist(browser, browaliashead);   /* user-requested aliases */
 
-    /* generate shortened name */
-    if ((c = strchr(browser, '/')) != NULL)
-      *c = '\0';
+    ++total_good_brows;
+    total_brow_pages += ispage;
+    total_brow_bytes += bytes;
 
-    /* special case */
-    if (strstr(browser, "Mosaic") != NULL || strstr(browser, "mosaic") != NULL)
-      strcpy(browser, "Mosaic");
+    /* add to full browser report */
+    if (Bq)
+      hashadd(Bhead, Bhashsize, browser, 1, bytes, ispage, last7q,
+	      &tempint, &tempint, &tempint, ON, OFF, OFF,
+	      (struct genstruct *)NULL, -1, 'B');
 
-    if (STREQ(browser, "Mozilla") && strstr(browser + 10, "ompatible") != NULL)
-      strcpy(browser, "Mozilla (compatible)");
+    if (bq) {
 
-    /* and add to browser summary */
-    hashadd(bhead, bhashsize, browser, 1, bytes, ispage, last7q, &tempint,
-	    &tempint, &tempint, ON, OFF, OFF, (struct genstruct *)NULL,
-	    -1, 'b');
+      /* generate shortened name */
+      if ((c = strchr(browser, '/')) != NULL)
+	*c = '\0';
+
+      /* special cases */
+      if (strstr(browser, "Mosaic") != NULL ||
+	  strstr(browser, "mosaic") != NULL)
+	strcpy(browser, "Mosaic");
+
+      if (STREQ(browser, "Mozilla") &&
+	  strstr(browser + 10, "ompatible") != NULL)
+	strcpy(browser, "Mozilla (compatible)");
+
+      /* and add to browser summary */
+      hashadd(bhead, bhashsize, browser, 1, bytes, ispage, last7q, &tempint,
+	      &tempint, &tempint, ON, OFF, OFF, (struct genstruct *)NULL,
+	      -1, 'b');
+    }
   }
 }
 
@@ -552,14 +564,17 @@ void adderr(char *errstr)
   int i;
   flag done = OFF;
 
-  for (i = 0; !done; i++) {
+  if (errstr != NULL && errstr[0] != '\0') { /* shouldn't occur, but be safe */
     strcpy(e1, errstr);
-    strcpy(e2, errs[i]);
-    if (strstr(strtoupper(e1), strtoupper(e2)) != NULL) {
-      done = ON;
-      errors[i]++;
-      if (debug >= 2 && i == NO_ERRS - 1)  /* unknown error type */
-	fprintf(stderr, "E: %s\n", errstr);
+    strtoupper(e1);
+    for (i = 0; !done; i++) {
+      strcpy(e2, errs[i]);
+      if (strstr(e1, strtoupper(e2)) != NULL) {
+	done = ON;
+	errors[i]++;
+	if (debug >= 2 && i == NO_ERRS - 1)  /* unknown error type */
+	  fprintf(stderr, "E: %s\n", errstr);
+      }
     }
   }
 }
@@ -590,7 +605,7 @@ void approxhostfill(char *space, unsigned int i)
 void approxhosthashadd(char *hostn, flag last7q)
 {
   extern char *approxhostspace, *approxhostspace7;
-  extern int approxhostsize;
+  extern size_t approxhostsize;
   extern int no_hosts, no_hosts7, no_new_hosts7;
   extern flag q7;
 
