@@ -1,4 +1,4 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** init.c; initialisation routines and declaration of global variables ***/
@@ -9,9 +9,17 @@
 /*** First declare all global variables. We choose to declare them in this
      file rather than analog.c because more are needed here. ***/
 
-char endayname[7][4] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
-char frdayname[7][4] = {"Dim", "Lun", "Mar", "Mer", "Jeu", "Ven", "Sam"};
-char dedayname[7][4] = {"Son", "Mon", "Die", "Mit", "Don", "Fre", "Sam"};
+/* lang: need new dayname and monthname here, possibly in HTML and non-HTML
+   versions. */
+char endayname[7][11] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+char frdayname[7][11] = {"Dim", "Lun", "Mar", "Mer", "Jeu", "Ven", "Sam"};
+char dedayname[7][11] = {"Son", "Mon", "Die", "Mit", "Don", "Fre", "Sam"};
+char itdayname[7][11] = {"Dom", "Lun", "Mar", "Mer", "Gio", "Ven", "Sab"};
+char esdayname[7][11] = {"Dom", "Lun", "Mar", "Mie", "Jue", "Vie", "Sab"};
+char dkhtmldayname[7][11] = {"s&oslash;n", "man", "tir", "ons", "tor", "fre",
+				 "l&oslash;r"};
+char dkdayname[7][11] = {"son", "man", "tir", "ons", "tor", "fre",
+				 "lor"};
 char enmonthname[12][12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
 			      "Aug", "Sep", "Oct", "Nov", "Dec"};
 char frmonthname[12][12] = {"Janv", "Fevr", "Mars", "Avri", " Mai", "Juin",
@@ -23,8 +31,14 @@ char demonthname[12][12] = {"Jan", "Feb", "Mar", "Apr", "Mai", "Jun", "Jul",
 			      "Aug", "Sep", "Okt", "Nov", "Dez"};
 char dehtmlmonthname[12][12] = {"Jan", "Feb", "M&auml;r", "Apr", "Mai", "Jun",
 				  "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"};
+char itmonthname[12][12] = {"Gen", "Feb", "Mar", "Apr", "Mag", "Giu",
+			      "Lug", "Ago", "Set", "Ott", "Nov", "Dic"};
+char esmonthname[12][12] = {"Ene", "Feb", "Mar", "Abr", "May", "Jun",
+			      "Jul", "Ago", "Sep", "Oct", "Nov", "Dic"};
+char dkmonthname[12][12] = {"jan", "feb", "mar", "apr", "maj", "jun",
+			      "jul", "aug", "sep", "okt", "nov", "dec"};
 char monthname[12][12];     /* set to whichever is appropriate */
-char dayname[7][4];
+char dayname[7][11];
 /* Note: month numbers run from 0 (Jan) to 11 (Dec) internally to this
    program (though not where month numbers are needed in user input and
    output) */
@@ -75,8 +89,8 @@ struct daily *firstD, *lastD;   /* ... months of daily data */
 struct hourly *firstH, *lastH;  /* ... days of hourly data */
 struct weekly *firstW, *lastW;  /* ... and to the first and last week */
 
-int no_urls;                  /* the number of distinct files found so far */
-int no_urls7;                 /* the number used in the last 7 days */
+int no_urls;               /* the number of distinct files found so far */
+int no_urls7;              /* the number used in the last 7 days */
 struct genstruct **rhead;  /* hash table of all files found */
 struct genstruct **ihead;  /* and one for all directories */
 struct genstruct **thead;  /* one for file types */
@@ -100,9 +114,9 @@ struct genstruct **fhead2;
 size_t rhashsize, ihashsize, thashsize, Shashsize, fhashsize, bhashsize;
 size_t Bhashsize, Ohashsize;  /* sizes of the hash tables */
 
-struct alias *filealiashead, *hostaliashead, *refaliashead, *routaliashead;
-struct alias *ioutaliashead, *Soutaliashead, *foutaliashead, *boutaliashead;
-struct alias *toutaliashead, *subdomshead;
+struct alias *filealiashead, *hostaliashead, *refaliashead, *browaliashead;
+struct alias *routaliashead, *ioutaliashead, *Soutaliashead, *foutaliashead;
+struct alias *boutaliashead, *toutaliashead, *subdomshead;
     /* subdomshead is not really an alias, but it's the right shape */
 struct include *wantfilehead, *wanthosthead, *wantrefhead, *wantreqhead;
 struct include *linkhead, *reflinkhead, *ispagehead, *noexpandhead;
@@ -201,7 +215,7 @@ int osortby, isortby, tsortby, Ssortby, rsortby, fsortby, bsortby, Bsortby;
 int dirlevel, pagewidth;
 char markchar;
 char *hostname, *hosturl;
-int debug;
+int debug, progressfreq, no_configs = 0;
 
 flag vblesonly;  /* commandline only. If ON, just print variables and exit */
 flag formq;      /* commandline only. If ON, just make a form interface */
@@ -209,8 +223,8 @@ flag formq;      /* commandline only. If ON, just make a form interface */
 /* We also declare the following pointers here. They are used to keep track
    of places in lists between functions in this file and init2.c */
 
-struct alias *filealiasp, *hostaliasp, *refaliasp, *routaliasp, *ioutaliasp;
-struct alias *Soutaliasp, *foutaliasp, *boutaliasp, *toutaliasp;
+struct alias *filealiasp, *hostaliasp, *refaliasp, *browaliasp, *routaliasp;
+struct alias *ioutaliasp, *Soutaliasp, *foutaliasp, *boutaliasp, *toutaliasp;
 struct alias *subdomp, *subdomtempp;
 struct include *wantfilep, *wanthostp, *wantrefp, *ispagep, *noexpandp;
 struct include *refexpandp, *wantreqp, *linkp, *reflinkp;
@@ -311,6 +325,7 @@ void defaults(void)
   lang = LANGUAGE;
   dialect = NONE;
   warnq = WARNINGS;
+  progressfreq = PROGRESSFREQ;
   graphical = GRAPHICAL;
   case_insensitive = (CASE == INSENSITIVE);
   munit = 0;
@@ -515,6 +530,9 @@ void init_structs(void)
   refaliashead = (struct alias *)xmalloc(sizeof(struct alias));
   refaliashead -> from[0] = '\0';
   refaliasp = refaliashead;
+  browaliashead = (struct alias *)xmalloc(sizeof(struct alias));
+  browaliashead -> from[0] = '\0';
+  browaliasp = browaliashead;
   routaliashead = (struct alias *)xmalloc(sizeof(struct alias));
   routaliashead -> from[0] = '\0';
   routaliasp = routaliashead;
@@ -562,17 +580,15 @@ void init_structs(void)
   refexpandp = refexpandhead;
   ispagehead = (struct include *)xmalloc(sizeof(struct include));
   ispagep = ispagehead;
-#ifdef DOS
-  strcpy(ispagep -> name, "*.htm");
+  strcpy(ispagep -> name, "*/");
   ispagep -> in = TRUE;
   ispagep -> next = (struct include *)xmalloc(sizeof(struct include));
   ispagep = ispagep -> next;
-#endif
   strcpy(ispagep -> name, "*.html");
   ispagep -> in = TRUE;
   ispagep -> next = (struct include *)xmalloc(sizeof(struct include));
   ispagep = ispagep -> next;
-  strcpy(ispagep -> name, "*/");
+  strcpy(ispagep -> name, "*.htm");
   ispagep -> in = TRUE;
   ispagep -> next = (struct include *)xmalloc(sizeof(struct include));
   ispagep = ispagep -> next;
@@ -746,23 +762,45 @@ void othervars(void)    /* vars initialised after command args and config */
     for (i = 0; i < 7; i++)
       strcpy(dayname[i], frdayname[i]);
     if (aq == HTML)
-    for (i = 0; i < 12; i++)
-      strcpy(monthname[i], frhtmlmonthname[i]);
+      for (i = 0; i < 12; i++)
+        strcpy(monthname[i], frhtmlmonthname[i]);
     else
-    for (i = 0; i < 12; i++)
-      strcpy(monthname[i], frmonthname[i]);
+      for (i = 0; i < 12; i++)
+        strcpy(monthname[i], frmonthname[i]);
   }
-  else {  /* lang == GERMAN */
+  else if (lang == GERMAN) {
     for (i = 0; i < 7; i++)
       strcpy(dayname[i], dedayname[i]);
     if (aq == HTML)
-    for (i = 0; i < 12; i++)
-      strcpy(monthname[i], dehtmlmonthname[i]);
+      for (i = 0; i < 12; i++)
+        strcpy(monthname[i], dehtmlmonthname[i]);
     else
-    for (i = 0; i < 12; i++)
-      strcpy(monthname[i], demonthname[i]);
+      for (i = 0; i < 12; i++)
+        strcpy(monthname[i], demonthname[i]);
   }
-  
+  else if (lang == SPANISH) {
+    for (i = 0; i < 7; i++)
+      strcpy(dayname[i], esdayname[i]);
+    for (i = 0; i < 12; i++)
+      strcpy(monthname[i], esmonthname[i]);
+  }
+  else if (lang == DANISH) {
+    if (aq == HTML)
+      for (i = 0; i < 7; i++)
+	strcpy(dayname[i], dkhtmldayname[i]);
+    else
+      for (i = 0; i < 7; i++)
+	strcpy(dayname[i], dkdayname[i]);
+    for (i = 0; i < 12; i++)
+      strcpy(monthname[i], dkmonthname[i]);
+  }
+  else {  /* lang == ITALIAN */
+    for (i = 0; i < 7; i++)
+      strcpy(dayname[i], itdayname[i]);
+    for (i = 0; i < 12; i++)
+      strcpy(monthname[i], itmonthname[i]);
+  }
+
   dirsufflength = (int)strlen(dirsuffix);
     
   if (rq || iq || tq) {
@@ -882,17 +920,18 @@ void othervars(void)    /* vars initialised after command args and config */
 
 void configline(char inputline[MAXLINELENGTH])  /* process one configline */
 {
-  enum {BADCOMMAND_, FILEALIAS_, HOSTALIAS_, REFALIAS_, REQOUTPUTALIAS_,
-	  DIROUTPUTALIAS_, HOSTOUTPUTALIAS_, REFOUTPUTALIAS_, BROWOUTPUTALIAS_,
-	  TYPEOUTPUTALIAS_, FILEINCLUDE_, FILEEXCLUDE_, FILEALLOW_,
-	  HOSTINCLUDE_, HOSTEXCLUDE_, HOSTALLOW_, REFINCLUDE_, REFEXCLUDE_,
-	  REFALLOW_, REQINCLUDE_, REQEXCLUDE_, REQALLOW_, LINKINCLUDE_,
-	  LINKEXCLUDE_, LINKALLOW_, REFLINKINCLUDE_, REFLINKEXCLUDE_,
-	  REFLINKALLOW_, FROM_, TO_, SUBDOMAIN_, WEEKBEGINSON_,
-	  APPROXHOSTSIZE_, ISPAGE_, ISNOTPAGE_, SEPCHAR_, REPSEPCHAR_,
-	  DECPOINT_, PRESEP_, REPORTORDER_, WITHARGS_, WITHOUTARGS_,
-	  REFWITHARGS_, REFWITHOUTARGS_, NOTSUBDOMAIN_, BASEURL_, DOMCOLS_,
-	  HOSTCOLS_, DIRCOLS_, TYPECOLS_, REQCOLS_, REFCOLS_, BROWCOLS_,
+  enum {BADCOMMAND_, FILEALIAS_, HOSTALIAS_, REFALIAS_, BROWALIAS_,
+	  REQOUTPUTALIAS_, DIROUTPUTALIAS_, HOSTOUTPUTALIAS_, REFOUTPUTALIAS_,
+	  BROWOUTPUTALIAS_, TYPEOUTPUTALIAS_, FILEINCLUDE_, FILEEXCLUDE_,
+	  FILEALLOW_, HOSTINCLUDE_, HOSTEXCLUDE_, HOSTALLOW_, REFINCLUDE_,
+	  REFEXCLUDE_, REFALLOW_, REQINCLUDE_, REQEXCLUDE_, REQALLOW_,
+	  LINKINCLUDE_,  LINKEXCLUDE_, LINKALLOW_, REFLINKINCLUDE_,
+	  REFLINKEXCLUDE_, REFLINKALLOW_, FROM_, TO_, SUBDOMAIN_,
+	  WEEKBEGINSON_, APPROXHOSTSIZE_, ISPAGE_, ISNOTPAGE_, SEPCHAR_,
+	  REPSEPCHAR_, DECPOINT_, PRESEP_, REPORTORDER_, WITHARGS_,
+	  WITHOUTARGS_, REFWITHARGS_, REFWITHOUTARGS_, NOTSUBDOMAIN_,
+	  BASEURL_, DOMCOLS_, HOSTCOLS_, DIRCOLS_, TYPECOLS_, REQCOLS_,
+	  REFCOLS_, BROWCOLS_,
 	  FULLBROWCOLS_, MONTHCOLS_, DAYCOLS_, FULLDAYCOLS_, WEEKCOLS_,
 	  HOURCOLS_, FULLHOURCOLS_, MONTHGRAPH_, DAYGRAPH_, FULLDAYGRAPH_,
 	  HOURGRAPH_, FULLHOURGRAPH_, WEEKGRAPH_, GRAPHICAL_, LOGFILE_,
@@ -914,13 +953,13 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
 	  CASE_, IMAGEDIR_, DIRSUFFIX_, MONTHLYUNIT_, HOURLYUNIT_,
 	  FULLHOURLYUNIT_, DAILYUNIT_, LOGOURL_, HEADERFILE_, FOOTERFILE_,
 	  FULLDAILYUNIT_, WEEKLYUNIT_, ALL_, GENERAL_, OUTPUT_, LANGUAGE_,
-	  DEBUG_, RAWBYTES_, UNCOMPRESS_, REQHASHSIZE_, DIRHASHSIZE_,
-	  TYPEHASHSIZE_, HOSTHASHSIZE_, REFHASHSIZE_, BROWHASHSIZE_,
-	  FULLBROWHASHSIZE_, SUBDOMHASHSIZE_,
+	  DEBUG_, PROGRESSFREQ_, RAWBYTES_, UNCOMPRESS_, REQHASHSIZE_,
+	  DIRHASHSIZE_, TYPEHASHSIZE_, HOSTHASHSIZE_, REFHASHSIZE_,
+	  BROWHASHSIZE_, FULLBROWHASHSIZE_, SUBDOMHASHSIZE_,
 #ifndef NODNS
 	  DNSHASHSIZE_, NUMLOOKUP_, DNSFILE_, DNSFRESHHOURS_,
 #endif
-	  OUTFILE_
+	  CONFIGFILE_, OUTFILE_
 }
   commandtype;   /* final '_'s to avoid clashes with #defines */
 
@@ -942,6 +981,8 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
       commandtype = HOSTALIAS_;
     else if (STREQ(string1, "REFALIAS"))
       commandtype = REFALIAS_;
+    else if (STREQ(string1, "BROWALIAS"))
+      commandtype = BROWALIAS_;
     else if (STREQ(string1, "REQOUTPUTALIAS"))
       commandtype = REQOUTPUTALIAS_;
     else if (STREQ(string1, "DIROUTPUTALIAS"))
@@ -1252,6 +1293,8 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
       commandtype = LANGUAGE_;
     else if (STREQ(string1, "DEBUG"))
       commandtype = DEBUG_;
+    else if (STREQ(string1, "PROGRESSFREQ"))
+      commandtype = PROGRESSFREQ_;
     else if (STREQ(string1, "RAWBYTES"))
       commandtype = RAWBYTES_;
     else if (STREQ(string1, "UNCOMPRESS"))
@@ -1282,6 +1325,8 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
     else if (STREQ(string1, "DNSFRESHHOURS"))
       commandtype = DNSFRESHHOURS_;
 #endif
+    else if (STREQ(string1, "CONFIGFILE"))
+      commandtype = CONFIGFILE_;
     else if (STREQ(string1, "OUTFILE"))
       commandtype = OUTFILE_;
     
@@ -1299,6 +1344,9 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
     case (REFALIAS_):
       configalias(string2, string3, &refaliasp, string1, inputline, rc);
       break;
+    case (BROWALIAS_):
+      configalias(string2, string3, &browaliasp, string1, inputline, rc);
+      break;
     case (REQOUTPUTALIAS_):
       configalias(string2, string3, &routaliasp, string1, inputline, rc);
       break;
@@ -2041,6 +2089,12 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
 	  lang = FRENCH;
 	else if (STREQ(string2, "GERMAN"))
 	  lang = GERMAN;
+	else if (STREQ(string2, "ITALIAN"))
+	  lang = ITALIAN;
+	else if (STREQ(string2, "SPANISH"))
+	  lang = SPANISH;
+	else if (STREQ(string2, "DANISH"))
+	  lang = DANISH;
 	else
 	  configwarning2(inputline);
       }
@@ -2048,6 +2102,9 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
     case (DEBUG_):
       configint(string2, &debug, string1, inputline, rc);	  
       break;
+    case (PROGRESSFREQ_):
+      configint(string2, &progressfreq, string1, inputline, rc);	  
+      break;
     case (RAWBYTES_):
       onoff(string2, &rawbytes, string1, inputline, rc);
       break;
@@ -2098,6 +2155,9 @@ void configline(char inputline[MAXLINELENGTH])  /* process one configline */
       configint(string2, &dnsfreshhours, string1, inputline, rc);
       break;
 #endif
+    case (CONFIGFILE_):
+      config(string2);
+      break;
     case (OUTFILE_):
       configstr(string2, outfile, string1, inputline, rc);
       break;
@@ -2114,6 +2174,13 @@ flag config(char *filename)
   if (STREQ(filename, "none"))
     return(0);
   else {
+    no_configs++;
+    if (no_configs > MAX_CONFIGS) {
+      fprintf(stderr, "%s: Error: More than %d configuration files:\n",
+	      commandname, MAX_CONFIGS);
+      fprintf(stderr, "  probably being called in circles: exiting\n");
+      exit(ERR);
+    }
     cf = fopenlog(filename, "configuration file", &ispipe);
     if (cf == NULL)
       return(1);
@@ -2282,13 +2349,20 @@ void commandline(int argc, char **argv)
 	else
 	  config(argv[i] + 2);
       }
+      else {
+	fprintf(stderr, "%s: Warning: Ignoring unknown option -g:\n",
+		commandname);
+	fprintf(stderr, "  see Readme.html for correct usage\n");
+	fprintf(stderr,
+		"  or go to http://www.statslab.cam.ac.uk/~sret1/analog/\n");
+      }
       break;
     case 'G':     /* default configuration file: done already */
       break;
     case 'h':     /* help */
       if (STREQ(argv[i] + 2, "elp")) {
-	fprintf(stderr, "For help see Readme.html, or\n");
-	fprintf(stderr, "  http://www.statslab.cam.ac.uk/~sret1/analog/analog/Readme.html\n");
+	fprintf(stderr, "For help see Readme.html, or ");
+	fprintf(stderr, "http://www.statslab.cam.ac.uk/~sret1/analog/\n");
 	exit(OK);
       }
       else        /* hourly summary */
@@ -2445,12 +2519,11 @@ void commandline(int argc, char **argv)
       clflag(&xq, argv[i]);
       break;
     default:
-      fprintf(stderr,
-      "%s: Error: Unknown option %s: see README.html for correct usage\n",
+      fprintf(stderr, "%s: Warning: Ignoring unknown option %s:\n",
 	      commandname, argv[i]);
+      fprintf(stderr, "  see Readme.html for correct usage\n");
       fprintf(stderr,
       "  or go to http://www.statslab.cam.ac.uk/~sret1/analog/\n");
-      exit(ERR);
     }
   }
 
@@ -2489,8 +2562,6 @@ void initialise(int argc, char **argv)
   defaults();   /* enter the defaults for all the variables
 		   (before possibly changing them). */
 
-  init_structs();   /* initialise all the structures (and pointers to them) */
-
 #ifdef MAC_EVENTS
   commandname = (char *)xmalloc(7);
   strcpy(commandname, "analog");
@@ -2499,6 +2570,8 @@ void initialise(int argc, char **argv)
   strcpy(commandname, argv[0]);
 #endif
 
+  init_structs();   /* initialise all the structures (and pointers to them) */
+
   commandline(argc, argv);  /* parse commandline. This also parses all the
 			       configuration files. */
 
@@ -2534,11 +2607,16 @@ void initialise(int argc, char **argv)
     lang = ENGLISH;  /* only affects days of the week */
     sepchar = '\0';
     repsepchar = '\0';
-    decpoint = '\0';  /* don't need this last one at the moment, but... */
+    decpoint = '.';  /* don't need this last one at the moment, but... */
   }
 
-  if (rsortby == BYPAGES)
+  if (rsortby == BYPAGES) {
+    fprintf(stderr,
+	    "%s: Warning: Cannot sort request report by page requests:\n",
+	    commandname);
+    fprintf(stderr, "  Sorting by requests instead.\n");
     rsortby = BYREQUESTS;
+  }
 
   if (case_insensitive) {
     alias_to_lower(filealiashead);
@@ -2653,9 +2731,15 @@ void printvbles(void)
 	  printf("US English\n");
 	else if (lang == FRENCH)
 	  printf("French\n");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  printf("German\n");
-      }
+	else if (lang == SPANISH)
+	  printf("Spanish\n");
+   	else if (lang == DANISH)
+	  printf("Danish\n");
+        else /* lang == ITALIAN */
+          printf("Italian\n");
+         }
       lang = ENGLISH;   /* So that whatincluded() goes in English */
       printf("The domains file is %s\n", domainsfile);
       if (((filep = fopen(domainsfile, "r")) == NULL) &&
@@ -2900,6 +2984,7 @@ void printvbles(void)
       pvalias("File", filealiashead);
       pvalias("Host", hostaliashead);
       pvalias("Referrer", refaliashead);
+      pvalias("Brower", browaliashead);
       
     }   /* end if aq != CACHE */
 
@@ -2914,14 +2999,18 @@ void printvbles(void)
       printf("\nExamining only times");
       if (fromtime.code > -INFINITY)
 	printf(" from %02d/%s/%d", fromtime.date,
-	       monthname[fromtime.monthno], fromtime.year);
+	       enmonthname[fromtime.monthno], fromtime.year);
       if (totime.code < INFINITY)
 	printf(" to %02d/%s/%d", totime.date,
-	       monthname[totime.monthno], totime.year);
+	       enmonthname[totime.monthno], totime.year);
       printf("\n");
     }
 
-    printf("\nDebug level is %d\n\n", debug);
+    printf("\nDebug level is %d\n", debug);
+    printf("Warnings are %s\n", warnq?"ON":"OFF");
+    if (progressfreq > 0)
+      printf("Progress report every %d logfile lines\n", progressfreq);
+    printf("\n");
 
   }  /* end not form */
 
