@@ -1,4 +1,4 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** init2.c; some initialisation functions not using many global variables */
@@ -130,13 +130,11 @@ void dnscachescan(void)
 {
   extern char *dnsfile;
   extern time_t dnsstaletime;
-  extern struct genstruct **Shead2;
-  extern size_t Shashsize;
   extern char *commandname;
-  extern flag hostmaskq, warnq, anywarns;
+  extern flag warnq, anywarns;
 
   FILE *cf;
-  long timec;
+  time_t timec;
   char number[MAXSTRINGLENGTH], alias[MAXSTRINGLENGTH];
   char inputline[MAXLINELENGTH];
   flag ispipe;
@@ -151,7 +149,7 @@ void dnscachescan(void)
 	  anywarns = ON;
 	}
       }
-      else if (timec > (long)dnsstaletime)
+      else if (timec > dnsstaletime)
 	dnshashadd(number, (alias[0] == '*')?NULL:alias, timec);
     }
     fcloselog(cf, dnsfile, "DNS cache file", ispipe);
@@ -237,12 +235,16 @@ void addwildlogs(struct loglist **p, char name[MAXSTRINGLENGTH],
   flag matched = FALSE;
 
   strcpy(dirname, name);
-  for (tempc = dirname + strlen(dirname); *tempc != DIRSEP &&
-       tempc != dirname; tempc--)
+  for (tempc = dirname + strlen(dirname) - 1; tempc >= dirname &&
+       *tempc != DIRSEP; tempc--)
     ;   /* run back to last DIRSEP if any */
-  if (tempc == dirname) {
+  if (tempc < dirname) {
     strcpy(filename, dirname);
+#ifdef MAC
+    dirname[0] = '\0';
+#else
     sprintf(dirname, ".%c", DIRSEP);
+#endif
   }
   else {
     strcpy(filename, tempc + 1);
@@ -320,7 +322,7 @@ void addwildlogs(struct loglist **p, char name[MAXSTRINGLENGTH],
 
 void addlogfile(struct loglist **p, char name[MAXSTRINGLENGTH],
 		char prefix[MAXSTRINGLENGTH], flag wildexpand)
-{
+{         /* wildexpand only used if NODIRENT off */
   char *tempstr;
   struct stringlist *tempstrlist, *tempslp;
 
@@ -572,12 +574,16 @@ void addwildstrlist(struct stringlist **p, char *name)
   flag matched = FALSE;
 
   strcpy(dirname, name);
-  for (tempc = dirname + strlen(dirname); *tempc != DIRSEP &&
-       tempc != dirname; tempc--)
-    ;
-  if (tempc == dirname) {
+  for (tempc = dirname + strlen(dirname) - 1; tempc >= dirname &&
+       *tempc != DIRSEP; tempc--)
+    ;      /* run back to last DIRSEP if any */
+  if (tempc < dirname) {
     strcpy(filename, dirname);
+#ifdef MAC
+    dirname[0] = '\0';
+#else
     sprintf(dirname, ".%c", DIRSEP);
+#endif
   }
   else {
     strcpy(filename, tempc + 1);
