@@ -1,8 +1,8 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** output.c; the output function. ***/
-/* See output2.c for subsidiary functions */
+/* See output2.c and output3.c for subsidiary functions */
 
 #include "analhea2.h"
 
@@ -13,7 +13,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    int errorder[])
 {
   extern char *outfile;
-  extern char dayname[7][4];
+  extern char dayname[7][11];
   extern char monthname[12][12];
   extern int monthlength[12];
   extern char *hostname;
@@ -28,7 +28,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
   extern int mrows, Hrows, Drows;
   extern flag xq, dq, Dq, Wq, hq, Hq, mq, Sq, rq, oq, iq, tq, fq, bq, Bq, cq;
   extern int eq;
-  extern int aq, lang, dialect;
+  extern int aq, lang, dialect, debug;
   extern struct include *linkhead, *reflinkhead;
   extern struct alias *routaliashead, *ioutaliashead, *Soutaliashead;
   extern struct alias *foutaliashead, *boutaliashead, *toutaliashead;
@@ -91,8 +91,11 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
   bprefix[0] = '\0';
   bprefix[1] = '\0';
 
-  if (STREQ(outfile, "stdout"))
+  if (STREQ(outfile, "stdout") || STREQ(outfile, "-")) {
     outf = stdout;
+    if (debug > 0)
+      fprintf(stderr, "F: Opening stdout as output file\n");
+  }
 
   else if ((outf = fopen(outfile, "a")) == NULL) {
     fprintf(stderr, "%s: Error: failed to open output file %s for writing.\n",
@@ -100,19 +103,31 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
     exit(ERR);  /* shouldn't get here because also tested at the beginning */
   }             /* (unless it's vanished in the meantime or something) */
 
+  else if (debug > 0)
+      fprintf(stderr, "F: Opening %s as output file\n", outfile);
+
   if (aq == CACHE)
     fprintf(outf,
 	    "CACHE type 2 produced by analog%s. Do not modify or delete!",
 	    VERSION);
   else {
     if (aq == HTML) {
+      fprintf(outf, "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n");
+      fprintf(outf, "<html>\n<head>\n");
+      fprintf(outf, "<meta name=\"GENERATOR\" content=\"analog%s\">\n",
+	      VERSION);
       if (lang == ENGLISH)
-	fprintf(outf, "<html>\n<head><title>Web Server Statistics for ");
+	fprintf(outf, "<title>Web Server Statistics for ");
       else if (lang == FRENCH)
-	fprintf(outf,
-		"<html>\n<head><title>Statistiques du Serveur Web pour ");
-      else /* lang == GERMAN */
-	fprintf(outf, "<html>\n<head><title>WWW-Server Statistiken fuer ");
+	fprintf(outf, "<title>Statistiques du Serveur Web pour ");
+      else if (lang == GERMAN)
+	fprintf(outf, "<title>WWW-Server Statistiken fuer ");
+      else if (lang == SPANISH)
+	fprintf(outf, "<title>Estad&iacute;sticas del Servidor Web de ");
+      else if (lang == DANISH)
+	fprintf(outf, "<title>Web Server Statistik for ");
+      else /* lang == ITALIAN */
+        fprintf(outf, "<title>Statistiche del Web Server per ");
       htmlfprintf(outf, hostname);
       fprintf(outf, "</title></head>\n");
       fprintf(outf, "<body>\n<h1><a NAME=\"Top\">");
@@ -126,8 +141,14 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  fprintf(outf, "Web Server Statistics</a> for ");
 	else if (lang == FRENCH)
 	  fprintf(outf, "Statistiques du Serveur Web</a> pour ");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "WWW-Server Statistiken</a> f&uuml;r ");
+	else if (lang == SPANISH)
+	  fprintf(outf, "Estad&iacute;sticas del Servidor Web</a> de ");
+	else if (lang == DANISH)
+ 	  fprintf(outf, "Web Server Statistik</a> for ");
+        else /* lang == ITALIAN */
+	  fprintf(outf, "Statistiche del Web Server</a> per ");
 	htmlfprintf(outf, hostname);
       }
       else {
@@ -135,8 +156,15 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  fprintf(outf, "Web Server Statistics</a> for <a HREF=\"");
 	else if (lang == FRENCH)
 	  fprintf(outf, "Statistiques du Serveur Web</a> pour <a HREF=\"");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "WWW-Server Statistiken</a> f&uuml;r <a HREF=\"");
+	else if (lang == SPANISH)
+	  fprintf(outf,
+		  "Estad&iacute;sticas del Servidor Web</a> de <a HREF=\"");
+	else if (lang == DANISH)
+	  fprintf(outf, "Web Server Statistik</a> for <a HREF=\"");
+        else /* lang == ITALIAN */
+          fprintf(outf, "Statistiche del Web Server</a> per <a HREF=\"");
 	htmlfprintf(outf, hosturl);
 	fprintf(outf, "\">");
 	htmlfprintf(outf, hostname);
@@ -153,10 +181,22 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	fprintf(outf, "Statistiques du Serveur Web pour %s\n", hostname);
 	fprintf(outf, "=================================");
       }
-      else {  /* lang == GERMAN */
+      else if (lang == GERMAN) {
 	fprintf(outf, "WWW-Server Statistiken fuer %s\n", hostname);
 	fprintf(outf, "============================");
       }
+      else if (lang == SPANISH) {
+	fprintf(outf, "Estadisticas del Servidor Web de %s\n", hostname);
+	fprintf(outf, "=================================");
+      }
+      else if (lang == DANISH) {
+	fprintf(outf, "Web Server Statistik for %s\n", hostname);
+	fprintf(outf, "===========================");
+      }
+      else {  /* lang == ITALIAN */
+        fprintf(outf, "Statistiche del Web Server per %s\n", hostname);
+        fprintf(outf, "===============================");
+      }
       for (i = (int)strlen(hostname); i > 0; i--)
 	fprintf(outf, "=");
       fprintf(outf, "\n");
@@ -269,10 +309,28 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr, "Rapport Mensuel");
 	    strcpy(langstr3, "     mois");
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Monatsbericht");
 	    strcpy(langstr3, "  Monate");
 	  }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Informe Mensual");
+	    strcpy(langstr3, "     mes");
+	  }
+	  else if (lang == DANISH) {
+	    if (aq == HTML) {
+	      strcpy(langstr, "M&aring;nedlig rapport");
+	      strcpy(langstr3, "   m&aring;ned");
+	    }
+	    else {
+	      strcpy(langstr, "Maanedlig rapport");
+	      strcpy(langstr3, "  maaned");
+	    }
+          }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Resoconto mesi");
+	    strcpy(langstr3, "    mese");
+	  }
 	  datehead(outf, maxreq, maxpages, maxbytes, mcols, &mgraph, "Monthly",
 		   langstr, langstr, langstr3, 'm', &munit, &fieldwidth,
 		   &pfieldwidth, &bfieldwidth, &graphwidth, &bdivider);
@@ -307,7 +365,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  for (i = mback?lasti:firsti; (mback?(i >= firsti):(i <= lasti)) &&
 	       j != 0; i += mback?(-1):1) {  /* thro months in chosen order */
 	    if (aq == PREFORMATTED)
-	      precols(outf, mcols, 'm', byq);
+	      precols(outf, mcols, 'm', byq, ON);
 	    else
 	      fprintf(outf, "%s %d: ", monthname[i], year);
 	    dateline(outf, mp -> reqs[i], mp -> pages[i], mp -> bytes[i],
@@ -358,10 +416,22 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr, "Rapport Hebdomadaire");
 	    strcpy(langstr3, " sem. com.");
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Wochenbericht");
 	    strcpy(langstr3, "Woch.beg.");
 	  }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Informe Semanal");
+	    strcpy(langstr3, "sem. com.");
+	  }
+	  else if (lang == DANISH) {
+	    strcpy(langstr, "Ugentlig rapport");
+	    strcpy(langstr3, "uge start.");
+	  }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Resoconto settimane");
+            strcpy(langstr3, "sett.com.");
+	  } 
 	  datehead(outf, maxreq, maxpages, maxbytes, Wcols, &Wgraph, "Weekly",
 		   langstr, langstr, langstr3, 'W', &Wunit, &fieldwidth,
 		   &pfieldwidth, &bfieldwidth, &graphwidth, &bdivider);
@@ -370,7 +440,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	finished = FALSE;
 	for (wp = Wback?lastW:firstW; !finished; wp = wp -> next) {
 	  if (aq == PREFORMATTED)
-	    precols(outf, Wcols, 'W', byq);
+	    precols(outf, Wcols, 'W', byq, ON);
 	  else
 	    fprintf(outf, "%2d/%s/%02d: ", wp -> start.date,
 		    monthname[wp -> start.monthno], wp -> start.year % 100);
@@ -417,11 +487,29 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr2, "R&eacute;sum&eacute; Quotidien");
 	    strcpy(langstr3, "jou");
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Tagesuebersicht");
 	    strcpy(langstr2, "Tages&uuml;bersicht");
 	    strcpy(langstr3, "Tag");
 	  }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Resumen Diario");
+	    strcpy(langstr2, "Resumen Diario");
+	    if (aq == HTML)
+	      strcpy(langstr3, "d&iacute;a");
+	    else
+	      strcpy(langstr3, "dia");
+	  }
+	  else if (lang == DANISH) {
+	    strcpy(langstr, "Daglig oversigt");
+	    strcpy(langstr2, "Daglig oversigt");
+	    strcpy(langstr3, "dag");
+          }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Sommario giorni della settimana");
+            strcpy(langstr2, "Sommario giorni della settimana");
+            strcpy(langstr3, "gio");
+	  }
 	  datehead(outf, maxreq, maxpages, maxbytes, dcols, &dgraph, "Daily",
 		   langstr, langstr2, langstr3, 'd', &dunit, &fieldwidth,
 		   &pfieldwidth, &bfieldwidth, &graphwidth, &bdivider);
@@ -430,7 +518,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	for(i = 0; i <= 6; i++) {
 	  j = (weekbeginson + i) % 7;
 	  if (aq == PREFORMATTED)
-	    precols(outf, dcols, 'd', byq);
+	    precols(outf, dcols, 'd', byq, ON);
 	  else
 	    fprintf(outf, "%s: ", dayname[j]);
 	  dateline(outf, dreq[j], dpag[j], dbytes[j], dcols, dgraph,
@@ -506,9 +594,21 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr, "Rapport Quotidien");
 	    strcpy(langstr3, "      date");  /* NB 1 char longer than Eng */
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Tagesbericht");
 	    strcpy(langstr3, "    Datum");
+          }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Informe Diario");
+	    strcpy(langstr3, "    fecha");
+	  }
+	  else if (lang == DANISH) {
+	    strcpy(langstr, "Daglig rapport");
+	    strcpy(langstr3, "     dato");
+	  }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Resoconto giorni");
+            strcpy(langstr3, "     data");
 	  }
 	  datehead(outf, maxreq, maxpages, maxbytes, Dcols, &Dgraph,
 		   "FullDaily", langstr, langstr, langstr3, 'D', &Dunit,
@@ -546,7 +646,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  for (i = Dback?lasti:firsti; (Dback?(i >= firsti):(i <= lasti)) &&
 	       j != 0; i += Dback?(-1):1) { /* thro days in chosen order */
 	    if (aq == PREFORMATTED)
-	      precols(outf, Dcols, 'D', byq);
+	      precols(outf, Dcols, 'D', byq, ON);
 	    else if (lang == ENGLISH && dialect == US_ENGLISH)
 	      fprintf(outf, "%s/%2d/%02d: ", monthname[monthno], i + 1,
 		      year % 100);
@@ -653,11 +753,26 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr2, "Rapport Horaire");
 	    strcpy(langstr3, "      date:hr"); /* NB 1 char longer than Eng */
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Stuendlicher Bericht");
 	    strcpy(langstr2, "St&uuml;ndlicher Bericht");
 	    strcpy(langstr3, "    Datum:St");
 	  }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Informe Horario");
+	    strcpy(langstr2, "Informe Horario");
+	    strcpy(langstr3, "    fecha:hr");
+	  }
+	  else if (lang == DANISH) {
+	    strcpy(langstr, "Time rapport");
+	    strcpy(langstr2, "Time rapport");
+	    strcpy(langstr3, "     dato:kl");
+	  }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Resoconto ore");
+            strcpy(langstr2, "Resoconto ore");
+            strcpy(langstr3, "     data:hh");  /* NB 1 char longer than Eng */
+	  }
 	  if (aq != CACHE)
 	    datehead(outf, maxreq, maxpages, maxbytes, Hcols, &Hgraph,
 		     "FullHourly", langstr, langstr2, langstr3, 'H',
@@ -703,7 +818,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    }
 	    else {
 	      if (aq == PREFORMATTED)
-		precols(outf, Hcols, 'H', byq);
+		precols(outf, Hcols, 'H', byq, ON);
 	      else if (lang == ENGLISH && dialect == US_ENGLISH)
 		fprintf(outf, "%s/%2d/%02d:%02d: ", monthname[monthno], date,
 			year % 100, i);
@@ -782,11 +897,26 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    strcpy(langstr2, "R&eacute;sum&eacute; Horaire");
 	    strcpy(langstr3, "hr");
 	  }
-	  else { /* lang == GERMAN */
+	  else if (lang == GERMAN) {
 	    strcpy(langstr, "Stuendliche Uebersicht");
 	    strcpy(langstr2, "St&uuml;ndliche &Uuml;bersicht");
 	    strcpy(langstr3, "St");
 	  }
+	  else if (lang == SPANISH) {
+	    strcpy(langstr, "Resumen Horario");
+	    strcpy(langstr2, "Resumen Horario");
+	    strcpy(langstr3, "hr");
+	  }
+	  else if (lang == DANISH) {
+	    strcpy(langstr, "Time oversigt");
+	    strcpy(langstr2, "Time oversigt");
+	    strcpy(langstr3, "kl");
+	  }
+          else { /* lang == ITALIAN */
+            strcpy(langstr, "Sommario ore del giorno");
+            strcpy(langstr2, "Sommario ore del giorno");
+            strcpy(langstr3, "hh");
+	  }
 	  datehead(outf, maxreq, maxpages, maxbytes, hcols, &hgraph, "Hourly",
 		   langstr, langstr2, langstr3, 'h', &hunit, &fieldwidth,
 		   &pfieldwidth, &bfieldwidth, &graphwidth, &bdivider);
@@ -794,7 +924,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 
 	for(i = 0; i <= 23; i++) {
 	  if (aq == PREFORMATTED)
-	    precols(outf, hcols, 'h', byq);
+	    precols(outf, hcols, 'h', byq, ON);
 	  else
 	    fprintf(outf, "%2d: ", i);
 	  dateline(outf, hreq[i], hpag[i], hbytes[i], hcols, hgraph,
@@ -835,6 +965,34 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "Host");
 	  langgender = 'n';
 	}
+        else if (lang == ITALIAN) {
+          strcpy(langstr, "Resoconto host");
+          strcpy(langstr2, "host");
+          strcpy(langstr3, "host");
+          strcpy(langstr4, "host");
+          langgender = 'n';  /* special code to mean: use _gli_, not _i_ */
+	}
+	else if (lang == SPANISH && aq == HTML) {
+	  strcpy(langstr, "Informe de <i>Hosts</i>");
+	  /* NB Italics, as 'host' is not Spanish */
+	  strcpy(langstr2, "<i>host</i>");
+	  strcpy(langstr3, "<i>hosts</i>");
+	  strcpy(langstr4, "host");
+	  langgender = 'm';
+	}
+	else if (lang == SPANISH) { /* and aq != HTML */
+	  strcpy(langstr, "Informe de Hosts");
+	  strcpy(langstr2, "host");
+	  strcpy(langstr3, "hosts");
+	  strcpy(langstr4, "host");
+	  langgender = 'm';
+	}
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Host rapport");
+	  strcpy(langstr2, "host");
+	  strcpy(langstr3, "hosts");
+	  strcpy(langstr4, "host");
+	}
 	else if (aq == HTML) { /* and FRENCH */
 	  strcpy(langstr, "Rapport des h&ocirc;tes");
 	  strcpy(langstr2, "h&ocirc;te");
@@ -853,7 +1011,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	       Ssortby, Sminreqstr, Sminpagestr, Sminbytestr, Smaxreqs,
 	       Smaxpages, Smaxbytes, Scols, "Host", langstr, langstr2,
 	       langstr3, langstr4, langgender, 'S', Ssortby == ALPHABETICAL,
-	       byq, NULL, Soutaliashead, "");
+	       byq, ON, NULL, Soutaliashead, "");
       }
       break;
     
@@ -873,6 +1031,26 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "Verzeichnis");
 	  langgender = 'n';
 	}
+        else if (lang == ITALIAN) {
+          strcpy(langstr," Resoconto direttori");
+          strcpy(langstr2, "direttorio");
+          strcpy(langstr3, "direttori");
+          strcpy(langstr4, "direttorio");
+          langgender = 'm';
+        }
+	else if (lang == SPANISH) {
+	  strcpy(langstr, "Informe de Directorios");
+	  strcpy(langstr2, "directorio");
+	  strcpy(langstr3, "directorios");
+	  strcpy(langstr4, "directorio");
+	  langgender = 'm';
+	}
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Directory rapport");
+	  strcpy(langstr2, "directory");
+	  strcpy(langstr3, "directories");
+	  strcpy(langstr4, "directory");
+	}
 	else if (aq == HTML) { /* and FRENCH */
 	  strcpy(langstr, "Rapport des r&eacute;pertoires");
 	  strcpy(langstr2, "r&eacute;pertoire");
@@ -890,7 +1068,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	genout(outf, isorthead, total_succ_reqs, total_page_reqs, total_bytes,
 	       isortby, iminreqstr, iminpagestr, iminbytestr, imaxreqs,
 	       imaxpages, imaxbytes, icols, "Directory", langstr, langstr2,
-	       langstr3, langstr4, langgender, 'i', FALSE, byq, NULL,
+	       langstr3, langstr4, langgender, 'i', FALSE, byq, ON, NULL,
 	       ioutaliashead, "");
       }
       break;
@@ -911,18 +1089,46 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "extension");
 	  langgender = 'f';
 	}
-	else { /* lang == GERMAN */
+	else if (lang == GERMAN) {
 	  strcpy(langstr, "Dateityp-Bericht");
 	  strcpy(langstr2, "Extension");
 	  strcpy(langstr3, "Extensions");
 	  strcpy(langstr4, "Extension");
 	  langgender = 'f';
 	}
+        else if (lang == ITALIAN) {
+          strcpy(langstr, "Resoconto tipi di file");
+          strcpy(langstr2, "estensione");
+          strcpy(langstr3, "estensioni");
+          strcpy(langstr4, "estensione");
+          langgender = 'f';
+        }
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Filtype rapport");
+	  strcpy(langstr2, "endelse");
+	  strcpy(langstr3, "endelser");
+	  strcpy(langstr4, "endelse");
+	  langgender = 'f';
+	}
+	else if (aq == HTML) { /* and SPANISH */
+	  strcpy(langstr, "Informe de Tipos de Fichero");
+	  strcpy(langstr2, "extensi&oacute;n");
+	  strcpy(langstr3, "extensi&oacute;nes");
+	  strcpy(langstr4, "extensi&oacute;n");
+	  langgender = 'f';
+	}
+	else { /* lang == SPANISH and aq != HTML */
+	  strcpy(langstr, "Informe de Tipos de Fichero");
+	  strcpy(langstr2, "extension");
+	  strcpy(langstr3, "extensiones");
+	  strcpy(langstr4, "extension");
+	  langgender = 'f';
+	}
 	genout(outf, tsorthead, total_succ_reqs, total_page_reqs,
 	       total_bytes, tsortby, tminreqstr, tminpagestr, tminbytestr,
 	       tmaxreqs, tmaxpages, tmaxbytes, tcols, "FileType", langstr,
 	       langstr2, langstr3, langstr4, langgender, 't', FALSE, byq,
-	       NULL, toutaliashead, "");
+	       ON, NULL, toutaliashead, "");
       }
       break;
 
@@ -942,6 +1148,29 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "Datei");
 	  langgender = 'f';
 	}
+        else if (lang == ITALIAN) {
+          strcpy(langstr, "Resoconto richieste");
+          strcpy(langstr2, "file richiesto");
+          strcpy(langstr3, "file richiesti");
+          strcpy(langstr4, "file");
+          langgender = 'm';
+        }
+	else if (lang == SPANISH) {
+	  strcpy(langstr, "Informe de Peticiones");
+	  strcpy(langstr2, "fichero solicitado");
+	  strcpy(langstr3, "ficheros solicitados");
+	  strcpy(langstr4, "fichero");
+	  langgender = 'm';
+	}
+	else if (lang == DANISH) {
+	  if (aq == HTML)
+	    strcpy(langstr, "Foresp&oslash;rgselsrapport");
+	  else
+	    strcpy(langstr, "Forespoergselsrapport");
+          strcpy(langstr2, "forespurgt fil");
+          strcpy(langstr3, "forespurgte filer");
+          strcpy(langstr4, "fil");
+	}
 	else if (aq == HTML) { /* and FRENCH */
 	  strcpy(langstr, "Rapport des requ&ecirc;tes");
 	  strcpy(langstr2, "fichier demand&eacute;");
@@ -959,7 +1188,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	genout(outf, rsorthead, total_succ_reqs, total_page_reqs, total_bytes,
 	       rsortby, rminreqstr, rminpagestr, rminbytestr, rmaxreqs,
 	       rmaxpages, rmaxbytes, rcols, "Request", langstr, langstr2,
-	       langstr3, langstr4, langgender, 'r', FALSE, byq,
+	       langstr3, langstr4, langgender, 'r', FALSE, byq, ON,
 	       (aq == HTML)?linkhead:NULL, routaliashead, baseurl);
       }
       break;
@@ -980,18 +1209,39 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "URL");
 	  langgender = 'f';
 	}
-	else { /* lang == FRENCH */
+	else if (lang == FRENCH) {
 	  strcpy(langstr, "Rapport des Appelants");
 	  strcpy(langstr2, "URL appelant");
 	  strcpy(langstr3, "URL appelants");
 	  strcpy(langstr4, "URL");
 	  langgender = 'm';
 	}
+	else if (lang == SPANISH) {
+	  strcpy(langstr, "Informe de Remitentes");
+	  strcpy(langstr2, "URL remitente");
+	  strcpy(langstr3, "URLs remitentes");
+	  strcpy(langstr4, "URL");
+	  langgender = 'm';
+	}
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Refererende adresserapport");
+	  strcpy(langstr2, "refererende URL");
+	  strcpy(langstr3, "refererende URLs");
+	  strcpy(langstr4, "URL");
+	  langgender = 'm';
+	}
+        else { /* lang == ITALIAN */
+          strcpy(langstr, "Resoconto provenienze");
+          strcpy(langstr2, "URL di provenienza");
+          strcpy(langstr3, "URL di provenienza");
+          strcpy(langstr4, "URL");
+          langgender = 'f';
+        }
 	genout(outf, fsorthead, total_good_refs, total_ref_pages,
 	       total_ref_bytes, fsortby, fminreqstr, fminpagestr, fminbytestr,
 	       fmaxreqs, fmaxpages, fmaxbytes, fcols, "Referrer", langstr,
 	       langstr2, langstr3, langstr4, langgender, 'f', FALSE, refbyq,
-	       (aq == HTML)?reflinkhead:NULL, foutaliashead, "");
+	       ON, (aq == HTML)?reflinkhead:NULL, foutaliashead, "");
       }
       break;
 
@@ -1014,7 +1264,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "Browser");
 	  langgender = 'n';
 	}
-	else { /* lang == FRENCH */
+	else if (lang == FRENCH) {
 	  if (aq == HTML)
 	    strcpy(langstr, "R&eacute;sum&eacute; des navigateurs");
 	  else
@@ -1024,11 +1274,39 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "navigateur");
 	  langgender = 'm';
 	}
+	else if (lang == SPANISH && aq == HTML) {
+    	  strcpy(langstr, "Resumen de <i>Browsers</i>");
+	  strcpy(langstr2, "<i>browser</i>");
+	  strcpy(langstr3, "<i>browsers</i>");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
+	else if (lang == SPANISH) {  /* and aq != HTML */
+	  strcpy(langstr, "Resumen de Browsers");
+	  strcpy(langstr2, "browser");
+	  strcpy(langstr3, "browsers");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Browser oversigt");
+	  strcpy(langstr2, "browser");
+	  strcpy(langstr3, "browsere");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
+        else { /* lang == ITALIAN */
+          strcpy(langstr, "Sommario browser");
+          strcpy(langstr2, "browser");
+          strcpy(langstr3, "browser");
+          strcpy(langstr4, "browser");
+          langgender = 'm';
+        }
 	genout(outf, bsorthead, total_good_brows, total_brow_pages,
 	       total_brow_bytes, bsortby, bminreqstr, bminpagestr,
 	       bminbytestr, bmaxreqs, bmaxpages, bmaxbytes, bcols, "Browser",
 	       langstr, langstr2, langstr3, langstr4, langgender, 'b', FALSE,
-	       browbyq, NULL, boutaliashead, "");
+	       browbyq, browbyq, NULL, boutaliashead, "");
       }
       break;
       
@@ -1048,18 +1326,46 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  strcpy(langstr4, "Browser");
 	  langgender = 'n';
 	}
-	else { /* lang == FRENCH */
+	else if (lang == FRENCH) {
 	  strcpy(langstr, "Rapport des navigateurs");
 	  strcpy(langstr2, "navigateur");
 	  strcpy(langstr3, "navigateurs");
 	  strcpy(langstr4, "navigateur");
 	  langgender = 'm';
 	}
+        else if (lang == ITALIAN) {
+          strcpy(langstr, "Resoconto browser");
+          strcpy(langstr2, "browser");
+          strcpy(langstr3, "browser");
+          strcpy(langstr4, "browser");
+          langgender = 'm';
+        }
+	else if (lang == DANISH) {
+	  strcpy(langstr, "Browser rapport");
+	  strcpy(langstr2, "browser");
+	  strcpy(langstr3, "browsere");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
+	else if (aq == HTML) { /*  and SPANISH */
+	  strcpy(langstr, "Informe de <i>Browsers</i>");
+	  strcpy(langstr2, "<i>browser</i>");
+	  strcpy(langstr3, "<i>browsers</i>");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
+	else { /* lang == SPANISH and aq != HTML */
+	  strcpy(langstr, "Informe de Browsers");
+	  strcpy(langstr2, "browsers");
+	  strcpy(langstr3, "browser");
+	  strcpy(langstr4, "browser");
+	  langgender = 'm';
+	}
 	genout(outf, Bsorthead, total_good_brows, total_brow_pages,
 	       total_brow_bytes, Bsortby, Bminreqstr, Bminpagestr,
 	       Bminbytestr, Bmaxreqs, Bmaxpages, Bmaxbytes, Bcols,
 	       "FullBrowser", langstr, langstr2, langstr3, langstr4,
-	       langgender, 'B', FALSE, browbyq, NULL, NULL, "");
+	       langgender, 'B', FALSE, browbyq, browbyq, NULL, NULL, "");
       }
       break;
 
@@ -1087,18 +1393,31 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  fprintf(outf, "\n\n<hr>\n<i>This analysis was produced by <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a>.\n", VERSION);
 	else if (lang == FRENCH)
 	  fprintf(outf, "\n\n<hr>\n<i>Ces statistiques sont produites par <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a>.\n", VERSION);
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "\n\n<hr>\n<i>Diese Statistik wurde mit <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a> erstellt.\n", VERSION);
+	else if (lang == SPANISH)
+	  fprintf(outf, "\n\n<hr>\n<i>Estad&iacute;sticas generadas por <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a>.\n", VERSION);
+	else if (lang == DANISH)
+	  fprintf(outf, "\n\n<hr>\n<i>Denne analyse blev produceret af <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a>.\n", VERSION);
+        else /* lang == ITALIAN */
+          fprintf(outf, "\n\n<hr>\n<i>Queste statistiche sono state prodotte da <a HREF=\"http://www.statslab.cam.ac.uk/~sret1/analog/\">analog%s</a>.\n", VERSION);
       }
       else if (lang == ENGLISH)
 	fprintf(outf, "This analysis was produced by analog%s.\n", VERSION);
       else if (lang == FRENCH)
 	fprintf(outf, "Ces statistiques sont produites par analog%s.\n",
 		VERSION);
-      else /* lang == GERMAN */
+      else if (lang == GERMAN)
         fprintf(outf, "Diese Statistik wurde mit analog%s erstellt.\n",
 		VERSION);
-      
+      else if (lang == SPANISH)
+	fprintf(outf, "Estadisticas generadas por analog%s.\n",
+		VERSION);
+      else if (lang == DANISH)
+	fprintf(outf, "Denne analyse blev produceret af analog%s.\n", VERSION);
+      else /* lang == ITALIAN */
+        fprintf(outf, "Queste statistiche sono state prodotte da analog%s.\n", VERSION);
+
       time(&stoptime);
   
       stoptime -= starttime;   /* so now measures elapsed time */
@@ -1110,16 +1429,27 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 		    "<br><b>Running time:</b> Less than 1 second.</i>\n");
 	  else if (lang == FRENCH)
 	    fprintf(outf, "<br><b>Temps d'ex&eacute;cution :</b> Moins d'une seconde.</i>\n");
-	  else /* lang == GERMAN */
+	  else if (lang == GERMAN)
 	    fprintf(outf, "<br><b>Laufzeit:</b> Unter einer Sekunde.</i>\n");
-
+	  else if (lang == SPANISH)
+	    fprintf(outf, "<br><b>Tiempo de ejecuci&oacute;n:</b> Menos de 1 segundo.</i>\n");
+	  else if (lang == DANISH)
+	    fprintf(outf, "<br><b>Analysetid:</b> Under 1 sekund.</i>\n");
+          else /* lang == ITALIAN */
+            fprintf(outf, "<br><b>Tempo di esecuzione:</b> Meno di un secondo.</i>\n");
 	}
 	else if (lang == ENGLISH)
 	  fprintf(outf, "Running time: Less than 1 second.\n");
 	else if (lang == FRENCH)
 	  fprintf(outf, "Temps d'execution : Moins d'une seconde.\n");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "Laufzeit: Unter einer Sekunde.\n");
+	else if (lang == SPANISH)
+	  fprintf(outf, "Tiempo d ejecucion: Menos de 1 segundo.\n");
+	else if (lang == DANISH)
+	  fprintf(outf, "Analysetid: Under 1 sekund.\n");
+        else /* lang == ITALIAN */
+          fprintf(outf, "Tempo di esecuzione: Meno di un secondo.\n");
       }
 
       else if (stoptime < 60) {
@@ -1131,9 +1461,19 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    fprintf(outf,
 		    "<br><b>Temps d'ex&eacute;cution :</b> %ld seconde%s.</i>\n",
 		    stoptime, (stoptime == 1)?"":"s");
-	  else /* lang == GERMAN */
+	  else if (lang == GERMAN)
 	    fprintf(outf, "<br><b>Laufzeit:</b> %ld Sekunde%s.</i>\n",
                     stoptime, (stoptime == 1)?"":"n");
+	  else if (lang == SPANISH)
+	    fprintf(outf, "<br><b>Tiempo de ejecuci&oacute;n:</b> %ld segundo%s.</i>\n",
+                    stoptime, (stoptime == 1)?"":"s");
+	  else if (lang == DANISH)
+	    fprintf(outf, "<br><b>Analysetid:</b> %ld sekund%s.</i>\n",
+		    stoptime, (stoptime == 1)?"":"er");
+          else /* lang == ITALIAN */
+            fprintf(outf,
+		    "<br><b>Tempo di esecuzione:</b> %ld second%s.</i>\n",
+                    stoptime, (stoptime == 1)?"o":"i");
 	}
 	else if (lang == ENGLISH)
 	  fprintf(outf, "Running time: %ld second%s.\n",
@@ -1141,9 +1481,18 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	else if (lang == FRENCH)
 	  fprintf(outf, "Temps d'execution : %ld seconde%s.\n",
 		  stoptime, (stoptime == 1)?"":"s");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "Laufzeit: %ld Sekunde%s.\n",
 		  stoptime, (stoptime == 1)?"":"n");
+	else if (lang == SPANISH)
+	  fprintf(outf, "Tiempo de ejecucion: %ld segundo%s.\n",
+		  stoptime, (stoptime == 1)?"":"s");
+	else if (lang == DANISH)
+	  fprintf(outf, "Analysetid: %ld sekund%s.\n",
+		  stoptime, (stoptime == 1)?"":"er");
+        else /* lang == ITALIAN */
+          fprintf(outf, "Tempo di esecuzione: %ld second%s.</i>\n",
+                  stoptime, (stoptime == 1)?"o":"i");                  
       }
   
       else {
@@ -1156,10 +1505,22 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	    fprintf(outf, "<br><b>Temps d'ex&eacute;cution :</b> %ld minute%s, %ld seconde%s.</i>\n",
 		    stoptime / 60, (stoptime / 60 == 1)?"":"s",
 		    stoptime % 60, (stoptime % 60 == 1)?"":"s");
-	else /* lang == GERMAN */
-	  fprintf(outf, "<br><b>Laufzeit:</b> %ld Minute%s, %ld Sekunde%s.</i>\n",
+	  else if (lang == GERMAN)
+	    fprintf(outf, "<br><b>Laufzeit:</b> %ld Minute%s, %ld Sekunde%s.</i>\n",
                     stoptime / 60, (stoptime / 60 == 1)?"":"n",
                     stoptime % 60, (stoptime % 60 == 1)?"":"n");
+	  else if (lang == SPANISH)
+	    fprintf(outf, "<br><b>Tiempo de ejecuci&oacute;n:</b> %ld minuto%s, %ld segundo%s.</i>\n",
+		    stoptime / 60, (stoptime / 60 == 1)?"":"s",
+		    stoptime % 60, (stoptime % 60 == 1)?"":"n");
+	  else if (lang == DANISH)
+	    fprintf(outf, "<br><b>Analysetid:</b> %ld minut%s, %ld sekund%s.</i>\n",
+		    stoptime / 60, (stoptime / 60 == 1)?"":"ter",
+		    stoptime % 60, (stoptime % 60 == 1)?"":"er");
+          else /* lang == ITALIAN */
+            fprintf(outf, "<br><b>Tempo di esecuzione:</b> %ld minut%s, %ld second%s.</i>\n",
+                    stoptime / 60, (stoptime / 60 == 1)?"o":"i",
+                    stoptime % 60, (stoptime % 60 == 1)?"o":"i");
 	}
 	else if (lang == ENGLISH)
 	  fprintf(outf, "Running time: %ld minute%s, %ld second%s.\n",
@@ -1169,10 +1530,22 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
 	  fprintf(outf, "Temps d'execution : %ld minute%s, %ld seconde%s.\n",
 		  stoptime / 60, (stoptime / 60 == 1)?"":"s",
 		  stoptime % 60, (stoptime % 60 == 1)?"":"s");
-	else /* lang == GERMAN */
+	else if (lang == GERMAN)
 	  fprintf(outf, "Laufzeit: %ld Minute%s, %ld Sekunde%s.\n",
                   stoptime / 60, (stoptime / 60 == 1)?"":"n",
                   stoptime % 60, (stoptime % 60 == 1)?"":"n");
+	else if (lang == SPANISH)
+	  fprintf(outf, "Tiempo de ejecucion: %ld minuto%s, %ld segundo%s.\n",
+                  stoptime / 60, (stoptime / 60 == 1)?"":"s",
+                  stoptime % 60, (stoptime % 60 == 1)?"":"s");
+	else if (lang == DANISH)
+	  fprintf(outf, "Analysetid: %ld minut%s, %ld sekund%s.\n",
+		  stoptime / 60, (stoptime / 60 == 1)?"":"ter",
+		  stoptime % 60, (stoptime % 60 == 1)?"":"er");
+        else /* lang == ITALIAN */
+          fprintf(outf, "Tempo di esecuzione: %ld minut%s, %ld second%s.\n",
+		  stoptime / 60, (stoptime / 60 == 1)?"o":"i",
+		  stoptime % 60, (stoptime % 60 == 1)?"o":"i");
       }
   
       if (!aq && (mq || Wq || dq || Dq || hq || oq || Sq || iq || rq)) {
@@ -1221,5 +1594,7 @@ void output(struct genstruct *rsorthead, struct genstruct *isorthead,
   }      /* end if aq != CACHE */
 
   fclose(outf);
+  if (debug > 0)
+    fprintf(stderr, "F: Closing %s\n", STREQ(outfile, "-")?"stdout":outfile);
 
 }
