@@ -1,4 +1,4 @@
-/*** analog 1.91beta ***/
+/*** analog 2.0 ***/
 /* Please read Readme.html, or http://www.statslab.cam.ac.uk/~sret1/analog/  */
 
 /*** sscanf.c; functions to replace sscanf(), which is far too slow, in
@@ -750,7 +750,7 @@ int sscanf_referrer(char *inputline, int *date, int *monthno, int *year,
 
 }
 
-/* The final function is due to Jason Linhart (jason@summary.net) */
+/* The function sscanf_webstar() is due to Jason Linhart (jason@summary.net) */
 
 #ifdef WEBSTAR
 /*
@@ -1046,3 +1046,401 @@ int sscanf_webstar(char *inputline, char hostn[MAXSTRINGLENGTH], int *date,
 }
 #endif
 
+/* The function sscanf_netpresenz() and associated functions are due to 
+   Nigel Perry (N.Perry@massey.ac.nz), September 1996 */
+
+/* Parse NetPresenz 4.x format logs
+
+   The format is:
+
+       time\tdate\tIP\ttype\tsession\taction\tfile\t\r
+       fullpath\r
+       Referer: path\r
+
+   Time may include am/pm. This code currently assumes date is dd/mm/yy,
+   this is probably incorrect and Mac international routines should be used
+   to parse both the time and date. 
+
+   Type is one of HTTP, Gopher, NetPresenz ... (admin entries), or user id for
+   FTP - the user id is an arbitrary string (users can type anything
+   in!). Session is used only for FTP and gives the number of the current user.
+
+   Action is one of get file, get dir, log in, log out, CGI, or error msg.
+   The log in/out are for FTP.
+
+   File is the short name, fullpath is the full (Mac) pathname, referer is the
+   full URL as supplied. Lines two and three are optional.
+
+   We will build an "agent" from the type & action fields, error from action
+   field. Login, log out and NetPresenz admin lines will be ignored.
+
+   There is no bytes sent field in the NetPresenz log, maybe this can be
+   generated from the full path at some point...
+*/
+
+#ifdef NETPRESENZ
+
+static const MatchPairs entryTypes[] =
+{ { "HTTP", NP_HTTP },
+    { "Gopher", NP_GOPHER },
+    { NULL, NP_FTP }
+};
+
+static const MatchPairs actionTypes[] =
+{ { "get file", NP_GETFILE },
+    { "get dir", NP_GETDIR },
+    { "log in", NP_LOGIN },
+    { "log out", NP_LOGOUT },
+    { "CGI", NP_CGI },
+    { NULL, NP_ERROR }
+};
+
+NP_type checkValue(char *typeStr, const MatchPairs table[])
+{
+  while(table->str != NULL)
+    { if(strcmp(table->str, typeStr) == 0) return table->value;
+      table++;
+    }
+  return table->value; /* last field contains default value to return */
+}
+
+/* the NetPresenz fields have to be treated differently as they are
+   matched by prefix only */
+
+NP_type checkType(char *typeStr)
+{ if(strncmp(typeStr, "NetPresenz", 10) == 0) return NP_ADMIN;
+  return checkValue(typeStr, entryTypes);
+}
+
+/* make up an agent value from the type and action fields */
+
+void build_agent(char agent[MAXSTRINGLENGTH], NP_type entryType,
+		 char *entryTypeStr, NP_type actionType, char *actionTypeStr)
+{ /* a simple algorithm for the moment */
+  char *ans;
+
+  switch(entryType)
+    { case NP_HTTP:
+	ans = actionType == NP_CGI ? "cgi" : "http";
+	break;
+      case NP_GOPHER:
+	ans = "gopher";
+	break;
+      case NP_FTP:
+	ans = "ftp";
+	break;
+      default:
+	ans = "unknown";
+	break;
+      }
+  strcpy(agent, ans);
+}
+
+/* the speed of sscanf is of concern, but I'm going to use a routine to read
+   in positive numbers - the cost is small.
+   The routine returns -1 on error, otherwise the value read in.
+   The argument pointer is updated.
+   */
+int readDigits(char **from)
+{
+  int ans;
+  char *cin = *from;
+
+  if(!isdigit(*cin)) return -1;
+  ans = *cin++ - '0';
+  
+  while(isdigit(*cin))
+    { ans *= 10;
+      ans += *cin++ - '0';
+    }
+  
+  *from = cin;
+  return ans;
+}
+
+/* scan a file name, obeying global flags
+   return 0 on error, 1 if ok
+   */
+int scan_path(char *inputline, char filename[MAXSTRINGLENGTH], int maxlen,
+	      struct include *expandhead)
+{
+  char *cin = inputline;
+  char *cout = filename;
+  int ix;
+
+  extern flag case_insensitive;
+
+  ix = 0;
+  while(*cin && *cin != '\n' &&
+        *cin != '?' && ix < maxlen)
+    { *cout++ = *cin++;
+      ix++;
+    }
+  *cout = '\0';
+  if (*cin == '?')
+    { if (case_insensitive)
+	strtolower(filename);  /* if no question mark, strtolower later */
+      if (!included(filename, UNSET, expandhead))
+	{ while (*cin && *cin != '\n' && ix < maxlen) /* read in args */
+	    { *cout++ = *cin++;
+	      ix++;
+	    }
+	  *cout = '\0';
+	}
+    }
+  if (*cin != '\n' && *cin != '?') return 0;    /* fouled up somewhere */
+
+  return 1;
+}
+
+int sscanf_netpresenz(FILE *logfile, char *inputline,
+		      char hostn[MAXSTRINGLENGTH], int *date,
+		      int *monthno, int *year, int *hr, int *min,
+		      char filename[MAXSTRINGLENGTH],
+		      char referer[MAXSTRINGLENGTH],
+		      char agent[MAXSTRINGLENGTH],
+		      int *code, char bytestr[16], size_t preflength)
+{
+  char *cin = inputline;
+  char *cout;
+  int peek;
+  char *entryTypeStr, *actionTypeStr;
+  char *entryEnd, *actionEnd;
+  NP_type entryType, actionType;
+  int ix;
+
+  extern struct include *noexpandhead, *refexpandhead;
+#ifdef MAC
+  Intl0Rec **intl_format;
+  int tmp;
+  static int dateOrder = -1;
+  
+  if (dateOrder<0) {
+    intl_format=(Intl0Rec **)IUGetIntl(0);
+    if (intl_format && *intl_format) dateOrder=(**intl_format).dateOrder;
+    else dateOrder=0;
+  }
+#endif
+  /* read time */
+  /* hour */
+  if((*hr = readDigits(&cin)) < 0) return 0;
+
+  if(*cin++ != ':') return 0;
+
+  /* min */
+  if((*min = readDigits(&cin)) < 0) return 0;
+
+  /* check for am/pm - we only check first character */
+  while(*cin == ' ') cin++; /* skip spaces */
+  switch(*cin)
+    { case 'a': case 'A':
+	if(*hr == 12) *hr = 0; /* 12am = 00:00 hours */
+	break;
+      case 'p': case 'P':
+	if(*hr != 12) *hr += 12; /* 12pm = 12:00 hours */
+	break;
+      case '\t':
+	break; /* no am/pm */
+      }
+
+  /* skip to next field */
+  while(*cin && (*cin != '\t')) cin++;
+  if(*cin++ != '\t') return 0;
+
+  /* date */
+  if((*monthno = readDigits(&cin)) < 0) return 0;
+  if(*cin++ != '/') return 0;
+  if((*date = readDigits(&cin)) < 0) return 0;
+  if(*cin++ != '/') return 0;
+  if((*year = readDigits(&cin)) < 0) return 0;
+#ifdef MAC
+  switch (dateOrder) {                    /* 0 and default - mdy */
+
+  case 1:                                                 /* 1 - dmy */
+    tmp = *monthno;
+    *monthno = *date;
+    *date=tmp;
+    break;
+  case 2:                                                 /* 2 - ymd */
+    tmp = *monthno;
+    *monthno = *date;
+    *date = *year;
+    *year=tmp;
+    break;
+  case 3:                                                 /* 3 - myd */
+    tmp = *year;
+    *year = *date;
+    *date=tmp;
+    break;
+  case 4:                                                 /* 4 - dym */
+    tmp = *monthno;
+    *monthno = *year;
+    *year = *date;
+    *date=tmp;
+    break;
+  case 5:                                                 /* 5 - ydm */
+    tmp = *monthno;
+    *monthno = *year;
+    *year=tmp;
+    break;
+  }
+#endif
+
+  /* adjust year */
+  *year += *year < 70 ? 2000 : 1900;
+  /* adjust month - this program starts at 0 */
+  *monthno -= 1;
+
+  /* skip to next field */
+  while(*cin && (*cin != '\t')) cin++;
+  if(*cin++ != '\t') return 0;
+
+  /* read in hostname */
+  ix = 0;
+  cout = hostn;
+  while(*cin != '\t' && *cin != '\0' && ix < (MAXSTRINGLENGTH - 1))
+    { *cout++ = *cin++;
+      ix++;
+    }
+  if (*cin++ != '\t') return(0);
+  *cout = '\0';
+  
+  /* now entry type - tab terminated string */
+  entryTypeStr = cin;
+  while(*cin && (*cin != '\t')) cin++;
+  if(*cin != '\t') return 0;
+
+  entryEnd = cin++;
+  /* so we can restore the tab on error */
+  *entryEnd = '\0';  /* write eos over tab */
+  entryType = checkType(entryTypeStr);
+
+  /* now let's be smart, if the entry type is NP_ADMIN rather than return
+     a bad line error, let's read a new line and start again. This stops
+     lines which are not really bad from being counted as such.
+     */
+  if(entryType == NP_ADMIN)
+    { int c;
+
+      if((c = fgetc(logfile)) == EOF) return(0);
+                   /* not much we can do in this case */
+      ungetc(c, logfile);  /* you never know it might be a one char line... */
+      fgets(inputline, MAXLINELENGTH, logfile);   /* won't fail -- >= 1 char */
+      return sscanf_netpresenz(logfile, inputline, hostn, date, monthno, year,
+			       hr, min, filename, referer, agent, code,
+			       bytestr, preflength);
+    }
+  /* we have no use for the session, skip it */
+  while(isdigit(*cin)) cin++;
+  if(*cin++ != '\t')
+    { *entryEnd = '\t';   /* return input to pristine state */
+      return 0;
+    }
+
+  /* now the action type - tab terminated string */
+  actionTypeStr = cin;
+  while(*cin && (*cin != '\t')) cin++;
+  if(*cin != '\t') return 0;
+
+  actionEnd = cin++;          /* so we can restore the tab on error */
+  *actionEnd = '\0';          /* write eos over tab */
+  actionType = checkValue(actionTypeStr, actionTypes);
+
+  /* similar to above, ignore log in/out */
+  if(actionType == NP_LOGIN || actionType == NP_LOGOUT)
+    { int c;
+
+      if((c = fgetc(logfile)) == EOF) return(0);
+      ungetc(c, logfile);
+      fgets(inputline, MAXLINELENGTH, logfile);
+      return sscanf_netpresenz(logfile, inputline, hostn, date, monthno, year,
+			       hr, min, filename, referer, agent, code,
+			       bytestr, preflength);
+    }
+
+  /* now the file - this is the short version which we might discard in
+     a moment if there is a full path following
+     */
+  cout = filename;
+  ix = 0;
+  /* 04/Oct/96 JTL <start> - catch missing file name */
+  if (!*cin || *cin=='\t') {
+        *cout++ = '/';
+        ix++;
+      }
+  else while(*cin && (*cin != '\t') &&
+	     (ix < (MAXLINELENGTH - 1 - preflength)))  {
+    if (*cin==':') {
+      *cout++ = '/';
+      ++cin;
+    }
+    else *cout++ = *cin++;    /* JTL <end> - and convert ':' to '/' */
+    ix++;
+  }
+  *cout = '\0';
+  /* add eos */
+
+  if(*cin != '\t')
+    { *entryEnd = *actionEnd = '\t';   /* return input to pristine state */
+      return 0;
+    }
+
+  /* we now have a complete first line, finish off */
+  build_agent(agent, entryType, entryTypeStr, actionType, actionTypeStr);
+  /* set ok/error based on actionType */
+  *code = actionType == NP_ERROR ? 404 : 200;
+  /* set bytes to zero - as a string... */
+  strcpy(bytestr, "0");
+  /* no referer yet */
+  *referer = '\0';
+
+  /* now try for optional lines - first the full pathname */
+  /* from this point on we always return 11 = referer & browser present,
+     latter code handles there being no refer by an empty string check.
+     This way we get a browser count which includes ftp & gopher
+     */
+  peek = fgetc(logfile);        /* peek at next char */
+  if(peek == EOF) return 11;
+  ungetc(peek, logfile);        /* put it back */
+  if(isdigit(peek)) return 11;
+
+  /* read in full path */
+  fgets(inputline, MAXLINELENGTH, logfile);  /* can't fail - ungetc above */
+  /* now copy it, obeying various flags - adapted from sscanf_common() */
+  *filename = '/';
+  /* leading / isn't there - add it */
+  /* note filename+1 & -2 to account for last line */
+  if(scan_path(inputline, filename+1, MAXLINELENGTH - 2 - preflength,
+	       noexpandhead) == 0)
+    return 0;
+  /* fouled up somewhere, and trashed our file :-( */
+
+  /* fix :'s in Mac path */
+  cin = filename;
+  while(*cin)
+    { if(*cin == ':') *cin = '/';
+      cin++;
+    }
+
+  /* ok, let's try for the referer... */
+  peek = fgetc(logfile);             /* peek at next char */
+  if(peek == EOF) return 11;
+  ungetc(peek, logfile);             /* put it back */
+  if(isdigit(peek)) return 11;
+
+  /* read in full line */
+  fgets(inputline, MAXLINELENGTH, logfile);
+
+  /* skip past "Referer: " */
+  cin = inputline;
+  while(*cin && *cin != ' ') cin++;
+  if(*cin++ != ' ') return 11;
+
+  /* copy referer */
+  if(scan_path(cin, referer, MAXLINELENGTH - 1, refexpandhead) == 0)
+    *referer ='\0';
+
+  return 11;
+  /* we got the lot! */
+}
+#endif /* NETPRESENZ */
